"""mainPage Configurator"""
# Author(s): Davide.De-Marchi@ec.europa.eu, Edoardo.Ramalli@ec.europa.eu
# Copyright © European Union 2024
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.

# Imports
from ipywidgets import widgets, HTML, Layout
import ipyvuetify as v
import json

# Vois imports
from vois import download
from vois.vuetify import settings, page, selectImage, Button, UploadImage, UploadJson, sliderFloat, ColorPicker, switch, selectSingle, tabs, sortableList, dialogGeneric, iconButton
from vois.vuetify import mainPage
from vois.templates import PageConfigurator


# Panels dimensioning
LEFT_WIDTH = '30vw'


#####################################################################################################################################################
# mainPage Configurator
#####################################################################################################################################################
class MainConfigurator(page.page):


    # Initialization
    def __init__(self, output, onclose=None, leftWidth=LEFT_WIDTH, **kwargs):
        super().__init__('mainPage', 'Visual configurator', output, onclose=onclose, copyrighttext='European Commission - Joint Research Centre',
                         left_back=True, **kwargs)

        # Initialize member variables
        self.leftWidth = 'calc(%s - 2px)'%leftWidth

        self.spacerX = v.Html(tag='div', style_='width: 10px; height:  0px;')
        self.spacerY = v.Html(tag='div', style_='width:  0px; height: 10px;')
        self.spacer  = v.Html(tag='div', style_='width: 10px; height: 10px;')
        
        self.debug = widgets.Output()
        
        self.updatePageEnabled = True
        
        
    #################################################################################################################################################
    # Create the page and returns the card widget where the content of the page must be displayed
    #################################################################################################################################################
    def create(self):
        super().create()
        
        # Cards for the panels
        st = 'border-radius: 0px; border-color: %s; border-width: 1px; overflow: auto;'%settings.color_first
        self.main_width  = 'calc(100vw - %s)'%self.leftWidth
        self.main_height = self.height
        self.cardLeft   = v.Card(flat=True, style_=st + 'border-right-width: 0px;', outlined=True, width=self.leftWidth, min_width=self.leftWidth, max_width=self.leftWidth, height=self.height)
        self.cardMain   = v.Card(flat=True, style_=st + 'display: flex; justify-content: center; align-items: center;', outlined=True, width=self.main_width, height=self.main_height, color='#eeeeee')
        
        # Creation of the contents for the panels
        self.createMain()
        self.createLeft()
        
        # Compose the panels
        self.card.children = [ widgets.HBox([self.cardLeft, self.cardMain]) ]
        
        # Set the logo
        self.logowidth  = 70
        self.logoappurl = mainPage.getLocalImageURL('./graphics/main.png')
        
        return self.card

    
    #################################################################################################################################################
    # Create the content of the left panel
    #################################################################################################################################################
    def createLeft(self):
        
        self.labelwidth  = 160
        self.sliderwidth = 140
        
        
        maxwidth = 'calc(%s - 20px)'%LEFT_WIDTH
        self.controls = v.Card(flat=True, width=maxwidth, min_width=maxwidth, max_width=maxwidth, class_='pa-3 pt-1 ma-0', style_='overflow: auto;')
        
        netwidth = 'calc(%s - 58px)'%LEFT_WIDTH
        titwidth = 'calc(%s - 210px)'%LEFT_WIDTH
        
        self.upload = UploadImage.UploadImage(self.output, width=620)
        
        self.tftitle = v.TextField(label='Application title:', autofocus=False, v_model=self.main.title, dense=True, color=settings.color_first, clearable=True, class_="pa-0 ma-0 mt-3 mr-6")
        self.text_color = ColorPicker(dark=False, color=self.main.text_color, width=50, height=30, rounded=False, on_change=self.text_colorChange,  offset_x=True, offset_y=False, color_theory_popup=True)
        ct  = v.Card(flat=True, children=[self.tftitle], width=titwidth, min_width=titwidth, max_width=titwidth)
        cc  = v.Card(flat=True, children=[widgets.HBox([PageConfigurator.label('Text color:', color='black', width=66), self.text_color, self.text_color.ctpopup.draw()])], class_='pa-0 ma-0 mt-4')
        cr1 = v.Card(flat=True, children=[widgets.HBox([ct, cc])], width=netwidth, max_width=netwidth)
        
        self.tfsubtitle = v.TextField(label='Application subtitle:', autofocus=False, v_model=self.main.subtitle, dense=True, color=settings.color_first, clearable=True, class_="pa-0 ma-0 mt-3 mr-3")
        
        
        self.tftitle.on_event(   'change',      self.titleChange)
        self.tftitle.on_event(   'click:clear', self.titleClear)
        self.tfsubtitle.on_event('change',      self.subtitleChange)
        self.tfsubtitle.on_event('click:clear', self.subtitleClear)
        
        self.titlesizepercent = sliderFloat.sliderFloat(self.main.titlesizepercent, text='Title size percent:', minvalue=0, maxvalue=200, maxint=200, showpercentage=True, decimals=0,
                                                        labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_titlesizepercent)
        self.subtitlesizepercent = sliderFloat.sliderFloat(self.main.subtitlesizepercent, text='Subtitle size percent:', minvalue=0, maxvalue=200, maxint=200, showpercentage=True, decimals=0,
                                                           labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_subtitlesizepercent)
        
        self.titleshadow = switch.switch(self.main.titleshadow, 'Add shadow around application title', inset=True, dense=True, onchange=self.titleshadowChange)
        self.titleshadow_color = ColorPicker(dark=False, color=self.main.titleshadow_color, width=50, height=30, rounded=False, on_change=self.titleshadow_colorChange,  offset_x=True, offset_y=False, color_theory_popup=True)
        
        self.applogo_image_url  = Button('Select image for the application logo', color_selected=settings.color_first, dark=settings.dark_mode, 
                                         text_weight=450, on_click=self.applogo_image_load, width=netwidth, height=40,
                                         tooltip='Click to select an image to use as application logo on the title bar', selected=True, rounded=settings.button_rounded)
        self.slogowidth = sliderFloat.sliderFloat(self.main.applogo_widthpercent, text='Application logo width %:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                 labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.logowidthChange)
        
        
        self.title_width   = sliderFloat.sliderFloat(self.main.titlebox_widthpercent, text='Title box width %:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                     labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_titlebox_widthpercent)
        self.title_height  = sliderFloat.sliderFloat(self.main.titlebox_heightpercent, text='Title box height %:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                     labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_titlebox_heightpercent)
        self.title_top     = sliderFloat.sliderFloat(self.main.titlebox_toppercent, text='Title box top %:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                     labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_titlebox_toppercent)
        self.title_opacity = sliderFloat.sliderFloat(self.main.titlebox_opacity*100.0, text='Title box opacity %:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                     labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_titlebox_opacity)
        self.title_border  = sliderFloat.sliderFloat(self.main.titlebox_border, text='Title box border (px):', minvalue=0.0, maxvalue=10.0, maxint=10, showpercentage=False, decimals=0,
                                                     labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_titlebox_border)
        
        self.background_image_url = Button('Select image for the application background', color_selected=settings.color_first, dark=settings.dark_mode, 
                                           text_weight=450, on_click=self.background_image_load, width=netwidth, height=40,
                                           tooltip='Click to select an image to use as application background', selected=True, rounded=settings.button_rounded)
        
        images = [{ "name": '%d'%x, "image": 'https://jeodpp.jrc.ec.europa.eu/services/shared/wallpapers/%d.jpg'%x, "max_width": 150, "max_height": 100 } for x in range(60)]
        self.background_image = selectImage.selectImage(images=images, selection=55, onchange=self.background_image_selected, width='380px',
                                                        label='Please select a stock image from the list', max_height=100, dense=True, outlined=True, clearable=True, margins="ma-0 mt-2 mr-1")

        self.filter_enabled = True
        self.filter_reset = Button('Reset all filters', color_selected=settings.color_first, dark=settings.dark_mode, 
                                    text_weight=450, on_click=self.onFilterReset, width=190, height=40,
                                    tooltip='Click to reset all filters for the background image', selected=True, rounded=settings.button_rounded)
        
        
        self.filter_blur = selectSingle.selectSingle('Blur:', ['0', '1', '2', '3', '4', '5', '6'], selection='0', width=80, clearable=False, onchange=self.onFilterChange)
        
        self.filter_grayscale = sliderFloat.sliderFloat(0, text='Grayscale:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                        labelwidth=self.labelwidth-70, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.onFilterChange)
        
        self.filter_brightness = sliderFloat.sliderFloat(100, text='Brightness:', minvalue=0.0, maxvalue=200.0, maxint=200, showpercentage=True, decimals=0,
                                                         labelwidth=self.labelwidth-70, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.onFilterChange)

        self.filter_contrast = sliderFloat.sliderFloat(100, text='Contrast:', minvalue=0.0, maxvalue=200.0, maxint=200, showpercentage=True, decimals=0,
                                                       labelwidth=self.labelwidth-70, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.onFilterChange)

        self.filter_saturate = sliderFloat.sliderFloat(100, text='Saturate:', minvalue=0.0, maxvalue=200.0, maxint=200, showpercentage=True, decimals=0,
                                                      labelwidth=self.labelwidth-70, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.onFilterChange)
        
        self.filter_hue = sliderFloat.sliderFloat(0, text='Hue:', minvalue=0.0, maxvalue=360.0, maxint=360, showpercentage=True, decimals=0,
                                                  labelwidth=self.labelwidth-70, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.onFilterChange)
        
        self.filter_opacity = sliderFloat.sliderFloat(100, text='Opacity:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                      labelwidth=self.labelwidth-70, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.onFilterChange)
        
        self.filter_sepia = sliderFloat.sliderFloat(0, text='Sepia:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                      labelwidth=self.labelwidth-70, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.onFilterChange)
        
        
        self.buttonbox_width  = sliderFloat.sliderFloat(self.main.buttonbox_widthpercent, text='Button box width %:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                        labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_buttonbox_widthpercent)
        self.buttonbox_height = sliderFloat.sliderFloat(self.main.buttonbox_heightpercent, text='Button box height %:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                        labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_buttonbox_heightpercent)
        self.buttonbox_top    = sliderFloat.sliderFloat(self.main.buttonbox_toppercent, text='Button box top %:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                        labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_buttonbox_toppercent)

        
        self.button_width  = sliderFloat.sliderFloat(self.main.button_widthpercent, text='Buttons width %:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                        labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_button_width)
        self.button_height = sliderFloat.sliderFloat(self.main.button_heightpercent, text='Buttons height %:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                        labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_button_height)
        self.button_elevation = sliderFloat.sliderFloat(self.main.button_elevation, text='Buttons elevation:', minvalue=0.0, maxvalue=10.0, maxint=10, showpercentage=False, decimals=0,
                                                        labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_button_elevation)
        self.button_opacity = sliderFloat.sliderFloat(self.main.titlebox_opacity*100.0, text='Buttons opacity %:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                      labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_button_opacity)
        self.button_titlesize = sliderFloat.sliderFloat(float(self.main.button_titlesize.replace('vh','')), text='Buttons title size:', minvalue=0.1, maxvalue=4.0, maxint=39, showpercentage=False, decimals=1,
                                                        labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_button_titlesize)
        self.button_subtitlesize = sliderFloat.sliderFloat(float(self.main.button_subtitlesize.replace('vh','')), text='Buttons subtitle size:', minvalue=0.1, maxvalue=4.0, maxint=39, showpercentage=False, decimals=1,
                                                           labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_button_subtitlesize)
        self.button_radius = sliderFloat.sliderFloat(float(self.main.button_radius.replace('px','')), text='Button radius (px):', minvalue=0.0, maxvalue=20.0, maxint=20, showpercentage=False, decimals=0,
                                                     labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_button_radius)
        
        self.blist = sortableList.sortableList(items=self.main.buttons, dark=False, allowNew=True, itemNew=self.buttonNew, itemContent=self.buttonDisplay, onchange=self.AssignOrderToButtons,
                                               onadded=self.buttonsUpdate, onremoved=self.buttonsUpdate, onmovedown=self.buttonsUpdate, onmoveup=self.buttonsUpdate)
        
        self.vois_show = switch.switch(self.main.vois_show, 'Show vois credits', inset=True, dense=True, onchange=self.vois_showChange)
        self.vois_opacity = sliderFloat.sliderFloat(self.main.vois_opacity*100.0, text='vois opacity:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                    labelwidth=90, sliderwidth=100, resetbutton=True, showtooltip=True, onchange=self.vois_opacityChange)
        if not self.main.vois_show: self.vois_opacity.slider.disabled = True

        self.tfcredits = v.TextField(label='Credits text:', autofocus=False, v_model=self.main.credits, dense=True, color=settings.color_first, clearable=True, class_="pa-0 ma-0 mt-3 mr-3")
        self.tfcredits.on_event('change',      self.creditsChange)
        self.tfcredits.on_event('click:clear', self.creditsClear)
        
        self.creditslogo_url  = Button('Select image for credits box', color_selected=settings.color_first, dark=settings.dark_mode, 
                                       text_weight=450, on_click=self.creditslogo_image_load, width=netwidth, height=40,
                                       tooltip='Click to select an image to use as credits logo on the bottom bar', selected=True, rounded=settings.button_rounded)
        
        self.credits_width   = sliderFloat.sliderFloat(self.main.creditbox_widthpercent, text='Credits box width %:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                       labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_creditbox_widthpercent)
        self.credits_height  = sliderFloat.sliderFloat(self.main.creditbox_heightpercent, text='Credits box height %:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                       labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_creditbox_heightpercent)
        self.credits_top     = sliderFloat.sliderFloat(self.main.creditbox_toppercent, text='Credits box top %:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                       labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_creditbox_toppercent)
        self.credits_opacity = sliderFloat.sliderFloat(self.main.creditbox_opacity*100.0, text='Credits box opacity %:', minvalue=0.0, maxvalue=100.0, maxint=100, showpercentage=True, decimals=0,
                                                       labelwidth=self.labelwidth, sliderwidth=self.sliderwidth, resetbutton=True, showtooltip=True, onchange=self.on_creditbox_opacity)
        
        self.tfdisclaimer = v.TextField(label='Disclaimer:', autofocus=False, v_model=self.main.disclaimer, dense=True, color=settings.color_first, clearable=True, class_="pa-0 ma-0 mt-3 mr-3")
        self.tfdisclaimer.on_event(   'change',      self.disclaimerChange)
        self.tfdisclaimer.on_event(   'click:clear', self.disclaimerClear)
        
        
        # Tabs to group controls
        self.card_title      = v.Card(flat=True)
        self.card_background = v.Card(flat=True)
        self.card_buttons    = v.Card(flat=True)
        self.card_credits    = v.Card(flat=True)
        self.card_vois       = v.Card(flat=True)
        self.tabsView = tabs.tabs(0, ['Title', 'Background', 'Buttons', 'Credits', 'Vois'],
                                  contents=[self.card_title, self.card_background, self.card_buttons, self.card_credits, self.card_vois],
                                  dark=False, onchange=None, row=True)

        
        self.buttOpen  = iconButton.iconButton(icon='mdi-folder-open',  onclick=self.onOpen,  tooltip='Load state from file',                 margins='pa-0 ma-0 mt-3 mr-2', color=settings.color_first)
        self.buttSave  = iconButton.iconButton(icon='mdi-content-save', onclick=self.onSave,  tooltip='Save current state to file',           margins='pa-0 ma-0 mt-3 mr-2', color=settings.color_first)
        self.buttCode  = iconButton.iconButton(icon='mdi-file-code',    onclick=self.onCode,  tooltip='Generate and download code in Python', margins='pa-0 ma-0 mt-3 mr-2', color=settings.color_first)
        self.buttReset = iconButton.iconButton(icon='mdi-backspace',    onclick=self.onReset, tooltip='Reset main page to default state',     margins='pa-0 ma-0 mt-3 mr-2', color=settings.color_first)
        
        self.card_title.children = [widgets.VBox([
            self.spacerY,
            self.spacerY,
            cr1,
            self.tfsubtitle,
            self.titlesizepercent.draw(),
            self.subtitlesizepercent.draw(),
            self.applogo_image_url,
            self.slogowidth.draw(),
            self.title_width.draw(),
            self.title_height.draw(),
            self.title_top.draw(),
            self.title_opacity.draw(),
            self.title_border.draw(),
            self.titleshadow.draw(),
            widgets.HBox([PageConfigurator.label('Shadow color:', color='black'), self.titleshadow_color, self.titleshadow_color.ctpopup.draw()])
        ])]
        
        self.card_background.children = [widgets.VBox([
            self.spacerY,
            self.spacerY,
            self.background_image_url,
            self.spacerY,
            self.spacerY,
            self.background_image,
            self.spacerY,
            self.spacerY,
            widgets.HBox([self.filter_reset, self.spacerX, self.spacerX, self.spacerX, self.filter_blur.draw()]), 
            self.filter_grayscale.draw(),
            self.filter_brightness.draw(),
            self.filter_contrast.draw(),
            self.filter_saturate.draw(),
            self.filter_hue.draw(),
            self.filter_sepia.draw(),
            self.filter_opacity.draw()
        ])]
        
        
        self.card_buttons.children = [widgets.VBox([
            self.spacerY,
            self.spacerY,
            self.buttonbox_width.draw(),
            self.buttonbox_height.draw(),
            self.buttonbox_top.draw(),
            self.button_width.draw(),
            self.button_height.draw(),
            self.button_elevation.draw(),
            self.button_opacity.draw(),
            self.button_titlesize.draw(),
            self.button_subtitlesize.draw(),
            self.button_radius.draw(),
            PageConfigurator.label('Buttons:'),
            self.blist.draw()
        ])]
        
        
        self.card_credits.children = [widgets.VBox([
            self.spacerY,
            self.spacerY,
            self.tfcredits,
            self.spacerY,
            self.creditslogo_url,
            self.spacerY,
            self.spacerY,
            self.tfdisclaimer,
            self.spacerY,
            self.spacerY,
            self.credits_width.draw(),
            self.credits_height.draw(),
            self.credits_top.draw(),
            self.credits_opacity.draw()
        ])]
        
        
        self.card_vois.children = [widgets.VBox([
            self.spacerY,
            self.spacerY,
            self.vois_show.draw(),
            self.spacerY,
            self.vois_opacity.draw()
        ])]
        
        self.controls.children = [ widgets.VBox([
            widgets.HBox([self.buttOpen.draw(), self.buttSave.draw(), self.buttCode.draw(), self.buttReset.draw()]),
            self.tabsView.draw()
        ])]
        
        self.cardLeft.children = [self.controls]

    
    
    #################################################################################################################################################
    # Create the content of the Main panel
    #################################################################################################################################################
    def createMain(self):

        self.main = mainPage.mainPage(background_image=55)
        self.updatePreview()
        
        
    #################################################################################################################################################
    # Open, Save Reset buttons
    #################################################################################################################################################
        
    # Load state from file
    def onOpen(self):
        uj = UploadJson.UploadJson(self.output, onOK=self.onSelectedState, required_attributes=['title', 'subtitle'], attributes_width=80)
        uj.show()
        
        
    # Called when the UploadJson dialog-box is closed with the OK button
    def onSelectedState(self, state):
        self.main.fromJson(state)
        self.updateWidgets()
        self.updatePreview()
            
            
        
    # Save current state to file
    def onSave(self):
        self.saveFilename = v.TextField(label='File name:', autofocus=True, v_model=self.main.title, dense=False, color=settings.color_first, clearable=False, class_="pa-0 ma-0 ml-3 mt-3 mr-3")
        dialogGeneric.dialogGeneric(title='Save and download current state...' , on_ok=self.onDoSaveState, 
                                    text='   ', color=settings.color_first, dark=settings.dark_mode,
                                    titleheight=40, width=600,
                                    show=True, addclosebuttons=True, addokcancelbuttons=True,
                                    fullscreen=False, content=[self.saveFilename], output=self.output)
        
    
    # Effective save and download of the current state
    def onDoSaveState(self):
        
        filename = self.saveFilename.v_model
        if filename[-5:] != '.json':
            filename += '.json'
        
        state = self.main.toJson()
    
        # Convert to string and download
        txt = json.dumps(state, indent=4)
        download.downloadText(txt, fileName=filename)
        
                
    # Save Python code
    def onCode(self):
        self.saveFilename = v.TextField(label='File name:', autofocus=True, v_model=self.main.title, dense=False, color=settings.color_first, clearable=False, class_="pa-0 ma-0 ml-3 mt-3 mr-3")
        dialogGeneric.dialogGeneric(title='Save and download Python code...' , on_ok=self.onDoSaveCode, 
                                    text='   ', color=settings.color_first, dark=settings.dark_mode,
                                    titleheight=40, width=600,
                                    show=True, addclosebuttons=True, addokcancelbuttons=True,
                                    fullscreen=False, content=[self.saveFilename], output=self.output)

    
    # Effective save and download of the Python code
    def onDoSaveCode(self):
        
        self.onDoSaveState()   # Saves <filename>.json
        
        with self.debug:
            jsonfile = self.saveFilename.v_model
            if jsonfile[-5:] != '.json':
                jsonfile += '.json'

            notebookfile = self.saveFilename.v_model
            notebookfile += '.ipynb'

            scallbacks = ''
            for b in self.main.buttons:
                if 'onclick' in b and isinstance(b['onclick'], str) and len(b['onclick']) > 0 and not ' ' in b['onclick']:
                    scallbacks += '''
def %s(*args):
    dialogMessage.dialogMessage(title='Clicked on button %s', titleheight=36,
                                text='Change this lines with your code',
                                addclosebuttons=True, show=True, width=400, output=output)
    
'''%(b['onclick'],b['title'])

            # Code generation
            txt = '''from vois.vuetify import settings
settings.dark_mode      = True
settings.color_first    = '#0d856d'
settings.color_second   = '#a0dcd0'
settings.button_rounded = False

from ipywidgets import widgets, HTML, Layout
from IPython.display import display
import json

from vois import cssUtils
from vois.vuetify import mainPage, dialogMessage

output = widgets.Output(layout=Layout(width='0px', height='0px'))
display(output)

cssUtils.allSettings(output)
cssUtils.switchFontSize(output,14)

m = mainPage.mainPage()

%s

with open('%s') as f:
    j = json.load(f)
    m.fromJson(j)
    
for b in m.buttons:
    if 'onclick' in b and isinstance(b['onclick'], str) and len(b['onclick']) > 0 and not ' ' in b['onclick']:
        b['onclick'] = globals()[b['onclick']]
    
m.open()'''%(scallbacks, jsonfile)

            lines = txt.split('\n')
            lines = ",\n".join(["\"%s\\n\""%x for x in lines])

            txt = '''{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "65d32196-829e-41b9-a4e4-bdb6381998e9",
   "metadata": {},
   "outputs": [],
   "source": [
    %s
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python [interapro_env]",
   "language": "python",
   "name": "conda-env-interapro_env-py"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.11.6"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
'''%lines
        
            # Download .ipynb file
            download.downloadText(txt, fileName=notebookfile)
            
            
            
    # Reset page to initial state
    def onReset(self):
        self.main = mainPage.mainPage(background_image=55)
        self.updateWidgets()
        self.updatePreview()
        
        
        
    #################################################################################################################################################
    # Update widgets reading members of self.main
    #################################################################################################################################################
    def updateWidgets(self):
        
        self.updatePageEnabled = False
        
        self.tftitle.v_model    = self.main.title
        self.text_color.color   = self.main.text_color
        self.tfsubtitle.v_model = self.main.subtitle
        
        self.titlesizepercent.value    = self.main.titlesizepercent
        self.subtitlesizepercent.value = self.main.subtitlesizepercent

        self.titleshadow.value       = self.main.titleshadow
        self.titleshadow_color.color = self.main.titleshadow_color
        
        self.slogowidth.value = self.main.applogo_widthpercent
        
        self.title_width.value    = self.main.titlebox_widthpercent
        self.title_height.value   = self.main.titlebox_heightpercent
        self.title_top.value      = self.main.titlebox_toppercent
        self.title_opacity.value  = self.main.titlebox_opacity*100.0
        self.title_border.value   = self.main.titlebox_border
        
        if isinstance(self.main.background_image, int):
            self.background_image.value = self.main.background_image
        else:
            self.background_image.value = -1

        self.filter_blur.value       = '0'
        self.filter_grayscale.value  = 0
        self.filter_brightness.value = 100
        self.filter_contrast.value   = 100
        self.filter_saturate.value   = 100
        self.filter_hue.value        = 0
        self.filter_opacity.value    = 100
        self.filter_sepia.value      = 0
                                                    
        filters = self.main.background_filter.strip().split(' ')
        for f in filters:
            if '(' in f and ')' in f:
                if 'blur' in f:
                    s = f.replace('blur','').replace('(','').replace(')','').strip().replace('px','')
                    if len(s) > 0:
                        self.filter_blur.value = str(int(s))

                if 'grayscale' in f:
                    s = f.replace('grayscale','').replace('(','').replace(')','').strip().replace('%','')
                    if len(s) > 0:
                        self.filter_grayscale.value = int(s)

                if 'brightness' in f:
                    s = f.replace('brightness','').replace('(','').replace(')','').strip().replace('%','')
                    if len(s) > 0:
                        self.filter_brightness.value = int(s)

                if 'contrast' in f:
                    s = f.replace('contrast','').replace('(','').replace(')','').strip().replace('%','')
                    if len(s) > 0:
                        self.filter_contrast.value = int(s)

                if 'saturate' in f:
                    s = f.replace('saturate','').replace('(','').replace(')','').strip().replace('%','')
                    if len(s) > 0:
                        self.filter_saturate.value = int(s)

                if 'hue-rotate' in f:
                    s = f.replace('hue-rotate','').replace('(','').replace(')','').strip().replace('deg','')
                    if len(s) > 0:
                        self.filter_hue.value = int(s)

                if 'sepia' in f:
                    s = f.replace('sepia','').replace('(','').replace(')','').strip().replace('%','')
                    if len(s) > 0:
                        self.filter_sepia.value = int(s)

                if 'opacity' in f:
                    s = f.replace('opacity','').replace('(','').replace(')','').strip().replace('%','')
                    if len(s) > 0:
                        self.filter_opacity.value = int(s)
            
        
        self.buttonbox_width.value     = self.main.buttonbox_widthpercent
        self.buttonbox_height.value    = self.main.buttonbox_heightpercent
        self.buttonbox_top.value       = self.main.buttonbox_toppercent
        
        self.button_width.value        = self.main.button_widthpercent
        self.button_height.value       = self.main.button_heightpercent
        self.button_elevation.value    = self.main.button_elevation
        self.button_opacity.value      = self.main.titlebox_opacity*100.0
        self.button_titlesize.value    = float(self.main.button_titlesize.replace('vh',''))
        self.button_subtitlesize.value = float(self.main.button_subtitlesize.replace('vh',''))
        self.button_radius.value       = float(self.main.button_radius.replace('px',''))
        
        self.blist.items = self.main.buttons
        
        self.vois_show.value = self.main.vois_show
        self.vois_opacity.value = self.main.vois_opacity*100.0
        if not self.main.vois_show: self.vois_opacity.slider.disabled = True

        self.tfcredits.value = self.main.credits
        
        self.credits_width.value   = self.main.creditbox_widthpercent
        self.credits_height.value  = self.main.creditbox_heightpercent
        self.credits_top.value     = self.main.creditbox_toppercent
        self.credits_opacity.value = self.main.creditbox_opacity*100.0
        
        self.tfdisclaimer.value = self.main.disclaimer
        
        self.updatePageEnabled = True
        
        
    #################################################################################################################################################
    # Management of events in widgets
    #################################################################################################################################################
        
    # Update the preview
    def updatePreview(self):
        self.cardMain.children = [self.main.preview()]
        
        
    
    # Change application title
    def titleChange(self, *args):
        if self.updatePageEnabled:
            if self.tftitle.v_model is None: name = ''
            else:                            name = self.tftitle.v_model
            self.main.title = name
            self.updatePreview()

    def titleClear(self, *args):
        if self.updatePageEnabled:
            self.main.title = ''
            self.updatePreview()

        
    # Change application subtitle
    def subtitleChange(self, *args):
        if self.updatePageEnabled:
            if self.tfsubtitle.v_model is None: name = ''
            else:                               name = self.tfsubtitle.v_model
            self.main.subtitle = name
            self.updatePreview()

    def subtitleClear(self, *args):
        if self.updatePageEnabled:
            self.main.subtitlesizepercent = ''
            self.updatePreview()

        
    def on_titlesizepercent(self, value):
        if self.updatePageEnabled:
            self.main.titlesizepercent = value
            self.updatePreview()
    
    def on_subtitlesizepercent(self, value):
        if self.updatePageEnabled:
            self.main.subtitlesizepercent = value
            self.updatePreview()

    # Change of the text_color
    def text_colorChange(self):
        if self.updatePageEnabled:
            self.main.text_color = self.text_color.color
            self.updatePreview()

        
    # titleshadow flag
    def titleshadowChange(self, flag):
        if self.updatePageEnabled:
            self.main.titleshadow = flag
            self.updatePreview()
        
    # titleshadow_color
    def titleshadow_colorChange(self,):
        if self.updatePageEnabled:
            self.main.titleshadow_color = self.titleshadow_color.color
            self.updatePreview()
    
    
    # Selection of the application image to load from the local machine
    def applogo_image_loaded(self, imageurl):
        if self.updatePageEnabled:
            self.main.applogo_url = imageurl
            self.updatePreview()

    # Click on the button to open the dialog to select the the application image
    def applogo_image_load(self, *args):
        if self.updatePageEnabled:
            self.upload.title = 'Select image for the application logo'
            self.upload.onOK  = self.applogo_image_loaded
            self.upload.show()

        
    # Change logo width
    def logowidthChange(self, width):
        if self.updatePageEnabled:
            self.main.applogo_widthpercent = int(width)
            self.updatePreview()
        
    # titlebox_widthpercent   
    def on_titlebox_widthpercent(self, value):
        if self.updatePageEnabled:
            self.main.titlebox_widthpercent = int(value)
            self.updatePreview()

    # titlebox_heightpercent
    def on_titlebox_heightpercent(self, value):
        if self.updatePageEnabled:
            self.main.titlebox_heightpercent = int(value)
            self.updatePreview()
        
    # titlebox_toppercent
    def on_titlebox_toppercent(self, value):
        if self.updatePageEnabled:
            self.main.titlebox_toppercent = int(value)
            self.updatePreview()
        
    # titlebox_opacity
    def on_titlebox_opacity(self, value):
        if self.updatePageEnabled:
            self.main.titlebox_opacity = value/100.0
            self.updatePreview()

    # titlebox_widthpercent   
    def on_titlebox_border(self, value):
        if self.updatePageEnabled:
            self.main.titlebox_border = int(value)
            self.updatePreview()
        
        
    # Selection of the backgound image to load from the local machine
    def background_image_loaded(self, imageurl):
        if self.updatePageEnabled:
            self.main.background_image = imageurl
            self.background_image.value = -1
            self.updatePreview()

    # Click on the button to open the dialog to select the the background image
    def background_image_load(self, *args):
        if self.updatePageEnabled:
            self.upload.title = 'Select image for the application background'
            self.upload.onOK  = self.background_image_loaded
            self.upload.show()
        
        
    # Selection of a stock background image
    def background_image_selected(self):
        if self.updatePageEnabled:
            if self.background_image.value >= 0:
                self.main.background_image = self.background_image.value
                self.updatePreview()
        
        
    # Filter reset
    def onFilterReset(self):
        if self.updatePageEnabled:
            self.filter_enabled = False
            self.filter_blur.value       = '0'
            self.filter_grayscale.value  = 0
            self.filter_brightness.value = 100
            self.filter_contrast.value   = 100
            self.filter_saturate.value   = 100
            self.filter_hue.value        = 0
            self.filter_opacity.value    = 100
            self.filter_sepia.value      = 0
            self.filter_enabled = True
            self.onFilterChange()
        
        
    # Filter change
    def onFilterChange(self, *args):
        if self.updatePageEnabled:
            if self.filter_enabled:
                filter = ''

                n = int(self.filter_blur.value)
                if n > 0: filter += 'blur(%dpx) '%n

                n = int(self.filter_grayscale.value)
                if n > 0: filter += 'grayscale(%d%%) '%n

                n = int(self.filter_brightness.value)
                if n != 100: filter += 'brightness(%d%%) '%n

                n = int(self.filter_contrast.value)
                if n != 100: filter += 'contrast(%d%%) '%n

                n = int(self.filter_saturate.value)
                if n != 100: filter += 'saturate(%d%%) '%n

                n = int(self.filter_hue.value)
                if n != 0: filter += 'hue-rotate(%ddeg) '%n

                n = int(self.filter_sepia.value)
                if n != 0: filter += 'sepia(%d%%) '%n

                n = int(self.filter_opacity.value)
                if n != 100: filter += 'opacity(%d%%) '%n

                self.main.background_filter = filter
                self.updatePreview()
        
        
    # buttonbox_widthpercent   
    def on_buttonbox_widthpercent(self, value):
        if self.updatePageEnabled:
            self.main.buttonbox_widthpercent = int(value)
            self.updatePreview()

    # buttonbox_heightpercent
    def on_buttonbox_heightpercent(self, value):
        if self.updatePageEnabled:
            self.main.buttonbox_heightpercent = int(value)
            self.updatePreview()
        
    # buttonbox_toppercent
    def on_buttonbox_toppercent(self, value):
        if self.updatePageEnabled:
            self.main.buttonbox_toppercent = int(value)
            self.updatePreview()

        
    # button_widthpercent
    def on_button_width(self, value):
        if self.updatePageEnabled:
            self.main.button_widthpercent = int(value)
            self.updatePreview()

    # button_heightpercent
    def on_button_height(self, value):
        if self.updatePageEnabled:
            self.main.button_heightpercent = int(value)
            self.updatePreview()

    # button_elevation
    def on_button_elevation(self, value):
        if self.updatePageEnabled:
            self.main.button_elevation = int(value)
            self.updatePreview()

    # button_opacity
    def on_button_opacity(self, value):
        if self.updatePageEnabled:
            self.main.button_opacity = value/100.0
            self.updatePreview()

    # button_titlesize
    def on_button_titlesize(self, value):
        if self.updatePageEnabled:
            self.main.button_titlesize = '%fvh'%value
            self.updatePreview()

    # button_subtitlesize
    def on_button_subtitlesize(self, value):
        if self.updatePageEnabled:
            self.main.button_subtitlesize = '%fvh'%value
            self.updatePreview()
            self.button_subtitlesize.draw(),
            
    # button_radius
    def on_button_radius(self, value):
        if self.updatePageEnabled:
            self.main.button_radius = '%dpx'%int(value)
            self.updatePreview()

        
    # Change of vois_show flag
    def vois_showChange(self, flag):
        if self.updatePageEnabled:
            self.main.vois_show = flag
            self.vois_opacity.slider.disabled = not self.main.vois_show
            self.updatePreview()
        
        
    # Change vois opacity
    def vois_opacityChange(self, opacity):
        if self.updatePageEnabled:
            self.main.vois_opacity = opacity / 100.0
            self.updatePreview()

        
    # Change credits text
    def creditsChange(self, *args):
        if self.updatePageEnabled:
            if self.tfcredits.v_model is None: name = ''
            else:                              name = self.tfcredits.v_model
            self.main.credits = name
            self.updatePreview()

    def creditsClear(self, *args):
        if self.updatePageEnabled:
            self.main.credits = ''
            self.updatePreview()
        
        
    # Selection of the credits image to load from the local machine
    def creditslogo_image_loaded(self, imageurl):
        if self.updatePageEnabled:
            self.main.creditslogo_url = imageurl
            self.updatePreview()

    # Click on the button to open the dialog to select the the credits image
    def creditslogo_image_load(self, *args):
        self.upload.title = 'Select image for the credits image'
        self.upload.onOK  = self.creditslogo_image_loaded
        self.upload.show()

        
    # Change disclaimer text
    def disclaimerChange(self, *args):
        if self.updatePageEnabled:
            if self.tfdisclaimer.v_model is None: name = ''
            else:                                 name = self.tfdisclaimer.v_model
            self.main.disclaimer = name
            self.updatePreview()

    def disclaimerClear(self, *args):
        if self.updatePageEnabled:
            self.main.disclaimer = ''
            self.updatePreview()


    # creditbox_widthpercent   
    def on_creditbox_widthpercent(self, value):
        if self.updatePageEnabled:
            self.main.creditbox_widthpercent = int(value)
            self.updatePreview()

    # creditbox_heightpercent
    def on_creditbox_heightpercent(self, value):
        if self.updatePageEnabled:
            self.main.creditbox_heightpercent = int(value)
            self.updatePreview()
        
        
    # creditbox_toppercent
    def on_creditbox_toppercent(self, value):
        if self.updatePageEnabled:
            self.main.creditbox_toppercent = int(value)
            self.updatePreview()
        
        
    # creditbox_opacity
    def on_creditbox_opacity(self, value):
        if self.updatePageEnabled:
            self.main.creditbox_opacity = value/100.0
            self.updatePreview()
        

    # Add a new button
    def buttonNew(self):
        eb = EditButton(self.output, index=len(self.main.buttons), newButton=True, onOK=self.AddNewButton)
        return None
    
    def AddNewButton(self, b):
        self.blist.doAddItem(b)
        self.updatePreview()
        
        
    # Display a button
    def buttonDisplay(self, item, index):
        
        def UpdateButton(b):
            self.main.buttons[b['index']] = b
            self.blist.items = self.main.buttons
            self.updatePreview()
            
        def onclick(*args):
            eb = EditButton(self.output, index=item['index'], title=item['title'], subtitle=item['subtitle'], tooltip=item['tooltip'], image=item['image'], onclick=item['onclick'], argument=item['argument'], newButton=False, onOK=UpdateButton)
        
        self.AssignOrderToButtons()
        lab = PageConfigurator.label(item['title'], class_='pa-0 ma-0 ml-2 mb-2', color='black')
        lab.on_event('click', onclick)
        return [lab]
        
        
    # Added or removed a button
    def buttonsUpdate(self, *args):
        if self.updatePageEnabled:
            self.updatePreview()
        
        
    # Called at each order change oand each remove of a button
    def AssignOrderToButtons(self):
        i = 0
        for b in self.main.buttons:
            b['index'] = i
            i += 1
        
        
#####################################################################################################################################################
# Create a New button or Edit info on a button
#####################################################################################################################################################
class EditButton():
    
    def __init__(self, output, index, title='', subtitle='', tooltip='', image='', onclick=None, argument=None, onOK=None, newButton=True):
        
        self.onOK = onOK
        self.upload = UploadImage.UploadImage(output, width=620)

        self.index = index
        
        self.image = image

        self.tit    = v.TextField(label='Title:',          autofocus=True,  v_model=title,    dense=True, color=settings.color_first, clearable=True, class_="pa-0 ma-0 mt-3")
        self.subtit = v.TextField(label='Subtitle:',       autofocus=False, v_model=subtitle, dense=True, color=settings.color_first, clearable=True, class_="pa-0 ma-0 mt-3")
        self.ttip   = v.TextField(label='Tooltip:',        autofocus=False, v_model=tooltip,  dense=True, color=settings.color_first, clearable=True, class_="pa-0 ma-0 mt-3")
        self.oncl   = v.TextField(label='On click call:',  autofocus=False, v_model=onclick,  dense=True, color=settings.color_first, clearable=True, class_="pa-0 ma-0 mt-3")
        self.arg    = v.TextField(label='Argument:',       autofocus=False, v_model=argument, dense=True, color=settings.color_first, clearable=True, class_="pa-0 ma-0 mt-3")

        b = Button('Select image for the button', color_selected=settings.color_first, dark=settings.dark_mode, 
                   text_weight=450, on_click=self.onSelectImage, width=570, height=40,
                   tooltip='Click to select an image to display on the button', selected=True, rounded=settings.button_rounded)

        c = v.Card(flat=True, children=[widgets.VBox([self.tit, self.subtit, self.ttip, self.oncl, self.arg, b])], class_='pa-4 ma-0')

        if newButton: dialogtitle = 'New button'
        else:         dialogtitle = 'Edit button'
        dlg = dialogGeneric.dialogGeneric(title=dialogtitle,
                                          text='',
                                          show=True, addclosebuttons=True, width=600,
                                          addokcancelbuttons=True, on_ok=self.AddedButton,
                                          fullscreen=False, content=[c], output=output)
        
        return None
        
        
    def onImageLoaded(self, imageurl):
        self.image = imageurl


    def onSelectImage(self):
        self.upload.title = 'Select image for the button'
        self.upload.onOK  = self.onImageLoaded
        self.upload.show()

    def AddedButton(self):
        self.button = {'title':    self.tit.v_model,
                       'subtitle': self.subtit.v_model,
                       'tooltip':  self.ttip.v_model,
                       'image':    self.image,
                       'onclick':  self.oncl.v_model,
                       'argument': self.arg.v_model,
                       'index':    self.index}
        
        if self.onOK is not None:
            self.onOK(self.button)
        


"""Initial page for an application"""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.

# Imports
from vois.vuetify import dialogGeneric, tooltip
from ipywidgets import widgets, Layout, HTML
from IPython.display import display
import ipyvuetify as v

import base64
from io import BytesIO
import os


######################################################################################################################################################
# Check if running in Voila: check environ variable SERVER_SOFTWARE
######################################################################################################################################################
def RunningInVoila():
    return os.environ.get('SERVER_SOFTWARE','jupyter').startswith('voila')
    #return not os.path.isdir("/eos/jeodpp/data/SRS/")

    
######################################################################################################################################################
# Given a local filepath of a PNG or JPEG image, returns the interned URL to use in a v.Img ipyvuetify widget
######################################################################################################################################################
def getLocalImageURL(filepath, imagetype='jpg'):
    with open(filepath, "rb") as f:
        buffered = BytesIO(f.read())
        img_str = base64.b64encode(buffered.getvalue()).decode('utf-8')
        return 'data:image/%s;base64,'%imagetype + img_str

    
######################################################################################################################################################
# Card that displays a clickable button
######################################################################################################################################################
class buttonCard(v.Html):

    def __init__(self,
                 width='400px',
                 height='160px',
                 margins='pa-0 ma-0 pl-3',
                 color='white',
                 dark=False,
                 ripple=False,
                 disabled=False,
                 elevation=3,
                 title='Title',
                 subtitle='Subtitle',
                 image='',
                 imagesize='160px',
                 on_click=None,
                 argument=None,
                 tooltiptext='',
                 textcolor='black',
                 titleweight=500,
                 subtitleweight=400,
                 titlesize='2vh',
                 subtitlesize='1.2vh',
                 button_radius='0px',
                 **kwargs):

        super().__init__(**kwargs)
        
        self.width     = width
        self.height    = height
        self.margins   = margins
        self.color     = color
        self.dark      = dark
        self.ripple    = ripple
        self.disabled  = disabled
        self.elevation = elevation
        self.title     = title
        self.subtitle  = subtitle
                 
        self.textcolor      = textcolor
        self.titleweight    = titleweight
        self.subtitleweight = subtitleweight
        self.titlesize      = titlesize
        self.subtitlesize   = subtitlesize
        self.button_radius  = button_radius
 
        self.image = image
        if len(image) == 0: self.imagesize = '0px'
        else:               self.imagesize = imagesize
        
        self.on_click = on_click
        self.argument = argument
        
        self.tooltiptext = tooltiptext
        
        
        self.card = v.Card(width=self.width, height=self.height, color=self.color, dark=self.dark, style_='border-radius: %s;'%self.button_radius,
                           ripple=self.ripple, disabled=self.disabled, elevation=self.elevation, class_=margins)
           
        t = v.CardTitle(children=[self.title], style_='color: %s; font-weight: %d; font-size: %s; padding: 0px; padding-left: 10px; padding-top: 10px;'%(self.textcolor,self.titleweight,self.titlesize))
        
        if len(self.subtitle) > 0:
            s = v.CardSubtitle(children=[self.subtitle], style_='color: %s; font-weight: %d; font-size: %s; padding: 0px; padding-left: 10px;'%(self.textcolor,self.subtitleweight,self.subtitlesize))
        else:
            s = v.Html(tag='div', children=[''])
        
        # Set the tooltip to title and subtitle
        t = tooltip.tooltip(self.tooltiptext,t)
        s = tooltip.tooltip(self.tooltiptext,s)
        
        if len(self.image) > 0:
            img = v.Img(src=self.image, contain=True, width=self.imagesize, min_width=self.imagesize, max_width=self.imagesize, height=self.height, max_height=self.height)
            img = tooltip.tooltip(self.tooltiptext,img)
            h = v.Html(tag='div', children=[t,s], style_='max-width: calc(%s - %s);'%(self.width,self.imagesize))
            self.card.children = [v.Row(justify='space-between', children=[h,img], style_='width: %s; max-width: %s;'%(self.width,self.width))]
        else:
            self.card.children = [t,s]
        
        if self.on_click is not None:
            self.card.on_event('click', self.__internal_onclick)
            
        self.tag = 'div'
        self.children = [self.card]
        
        
    # Click on the card
    def __internal_onclick(self, *args):
        if self.argument is None:
            self.on_click()
        else:
            self.on_click(self.argument)        



#####################################################################################################################################################
# Main page o the application
#####################################################################################################################################################
class mainPage():
    """
    Initial page of an application. The page contains a box for the title/subtitle/logo of the application, a box containing one or more buttons, each of them calling a callback function, and a box containing credits information. All these boxes are horizontally centered inside the page, while their vertical position can be customized (by default the title is on top, the buttons in central position and the credit box is at the bottom of the page).
        
    Parameters
    ----------
    title : str, optional
        Title of the page (default is 'Application main title')
    subtitle : str, optional
        Subtitle of the page (default is 'Subtitle to be displayed below the main title')
    applogo_url : str, optional
        Url of the application logo (default is a sample logo)
    applogo_widthpercent : float, optional
        Width of the area where the application logo is displayed in percentage of the screen (default is 20.0)
    titlesizepercent: int, optional
        Font size used for the main page title, in percentage compared to the standard dimension (default is 100, change to e.g. 80 for a smaller font, 120 for a bigger one)
    subtitlesizepercent: int, optional
        Font size used for the main page subtitle, in percentage compared to the standard dimension (default is 100, change to e.g. 80 for a smaller font, 120 for a bigger one)
    credits : str, optional
        Credits string to display inside the credit box on the bottom of the page (default is 'Unit T.4, Data Visualisation Team')
    creditslogo_url : str, optional
        Url of the image to display inside the credit box on the bottom of the page (default is 'https://jeodpp.jrc.ec.europa.eu/services/shared/pngs/TransparentJRC.png')
    text_color : str, optional
        Color to use for text (title, subtitle, credits, buttons text, etc.). Default is '#0e446e'.
    background_image : str or int, optional
        Image to use as fullscreen background. If an integer in the range [0,59] is passed, one of the sixty predefined wallpapers is used, otherwise any image URL can be used (default is 22)
    background_filter : str, optional
        CSS image filter to apply to the background image (default is ''). See https://developer.mozilla.org/en-US/docs/Web/CSS/filter for a list of available filters.
    vois_show : bool, optional
        Is True, a credit to the vois library is displayed in the top-right side of the page (default is True)
    vois_opacity : float, optional
        Opacity to apply to the vois logo, in case vois_show is set to True (default is 0.1)
    titlebox_widthpercent : float, optional
        Width of the top box containing the title, in percentage of the screen width. Default is 40.0.
    titlebox_heightpercent : float, optional
        Height of the box containing the title, in percentage of the screen height. Default is 24.0.
    titlebox_toppercent : float, optional
        Top position of the box containing the title, in percentage of the screen height, measured from the top of the page. Default is 12.0.
    titlebox_opacity : float, optional
        Opacity to apply to the box containing the title (default is 0.4)
    titlebox_border : int, optional
        Border width in pixels of the box containing the title (default is 2)
    buttonbox_widthpercent : float, optional
        Width of the box containing the buttons, in percentage of the screen width. Default is 80.0.
    buttonbox_heightpercent : float, optional
        Height of the box containing the buttons, in percentage of the screen height. Default is 50.0.
    buttonbox_toppercent : float, optional
        Top position of the box containing the buttons, in percentage of the screen height, measured from the top of the page. Default is 43.0.
    creditbox_widthpercent : float, optional
        Width of the box containing the credits, in percentage of the screen width. Default is 80.0.
    creditbox_heightpercent : float, optional
        Height of the box containing the credits, in percentage of the screen height. Default is 25.0.
    creditbox_toppercent : float, optional
        Top position of the box containing the credits, in percentage of the screen height, measured from the top of the page. Default is 74.0.
    creditbox_opacity : float, optional
        Opacity to apply to the box containing the credits (default is 0.0)
    button_widthpercent : float, optional
        Width of each of the buttons, in percentage of the screen width. Default is 22.0.
    button_heightpercent : float, optional
        Height of each of the buttons, in percentage of the screen height. Default is 10.0.
    button_elevation : int, optional
        Elevation in pixels to apply to the buttons (default is 6)
    button_opacity: float, optional
        Opacity to apply to the buttons (default is 0.5)
    button_titlesize: str, optional
        Font size to use for the buttons title (default is '2.0vh')
    button_subtitlesize: str, optional
        Font size to use for the buttons subtitle (default is '1.2vh')
    button_radius: str, optional
        Border radius for the buttons area (default is '0px' which means completely squared buttons)
    disclaimer: str, optional
        Text to display at the bottom of the creditbox (default is the empty string)

    Examples
    --------
    Creation of a main page with 6 buttons displaying random images from https://picsum.photos/::
        
        from vois.vuetify import mainPage
        from random import randrange
        
        def onclick1():
            print("Clicked Function 1")

        m = mainPage.mainPage(title='mainPage demo',
                              subtitle='Showcase how easy is to create a front page for an app',
                              credits="vois library development team",
                              titlebox_widthpercent=50, titlebox_opacity=0.2, titlebox_border=0,
                              vois_show=True, vois_opacity=0.1,
                              button_widthpercent=23, button_heightpercent=14, button_elevation=16, button_opacity=0.6,
                              background_image=55,
                              background_filter='blur(2px) brightness(1.2) contrast(0.7) sepia(0.05) saturate(1.2)',
                              creditbox_opacity=0,
                              text_color='#222222')

        m.addButton('Function 1',
                    subtitle='Launch calculation of ...',
                    tooltip='Tooltip text to display on hover',
                    image='https://picsum.photos/seed/%d/200/200'%randrange(1000),
                    onclick=onclick1)

        for i in range(2,7): m.addButton('Function %d'%i, image='https://picsum.photos/seed/%d/200/200'%randrange(1000))

        m.open()


    .. figure:: figures/mainPage.png
       :scale: 100 %
       :alt: mainPage widget

       Example of a mainPage
       

    """

    # Initialization
    def __init__(self,
                 title='Application main title',
                 subtitle='Subtitle to be shown below the main title',
                 titlesizepercent=100,
                 subtitlesizepercent=100,
                 applogo_url='https://jeodpp.jrc.ec.europa.eu/services/shared/pngs/sampleapplogo.png',
                 applogo_widthpercent=20.0,
                 credits='Unit T.4, Data Visualisation Team',
                 creditslogo_url='https://jeodpp.jrc.ec.europa.eu/services/shared/pngs/TransparentJRC.png',
                 text_color='#0e446e',
                 background_image=22,
                 background_filter='',
                 vois_show=True,
                 vois_opacity=0.1,
                 titlebox_widthpercent=40.0,
                 titlebox_heightpercent=24.0,
                 titlebox_toppercent=12.0,
                 titlebox_opacity=0.4,
                 titlebox_border=2,
                 titleshadow=True,
                 titleshadow_color='#ffffff',
                 buttonbox_widthpercent=80.0,
                 buttonbox_heightpercent=50.0,
                 buttonbox_toppercent=43.0,
                 creditbox_widthpercent=80.0,
                 creditbox_heightpercent=25.0,
                 creditbox_toppercent=74.0,
                 creditbox_opacity=0.,
                 button_widthpercent=22.0,
                 button_heightpercent=10.0,
                 button_elevation=6,
                 button_opacity=0.5,
                 button_titlesize='2.0vh',
                 button_subtitlesize='1.2vh',
                 button_radius='0px',
                 disclaimer='The contents of this viewer are intended solely for the use of the Commission and may not be reproduced, distributed, or communicated outside the Commission in any format without explicit prior written consent'):

        self.output = widgets.Output(layout=Layout(width='0px', height='0px'))
        
        self.title                   = title
        self.subtitle                = subtitle
        self.titlesizepercent        = titlesizepercent
        self.subtitlesizepercent     = subtitlesizepercent
        self.applogo_url             = applogo_url
        self.applogo_widthpercent    = applogo_widthpercent
        self.credits                 = credits
        self.creditslogo_url         = creditslogo_url
        self.text_color              = text_color
        self.background_image        = background_image
        self.background_filter       = background_filter
        self.vois_show               = vois_show
        self.vois_opacity            = vois_opacity

        self.titlebox_widthpercent   = titlebox_widthpercent
        self.titlebox_heightpercent  = titlebox_heightpercent
        self.titlebox_toppercent     = titlebox_toppercent
        self.titlebox_opacity        = titlebox_opacity
        self.titlebox_border         = titlebox_border
                         
        self.titleshadow             = titleshadow
        self.titleshadow_color       = titleshadow_color

        self.buttonbox_widthpercent  = buttonbox_widthpercent
        self.buttonbox_heightpercent = buttonbox_heightpercent
        self.buttonbox_toppercent    = buttonbox_toppercent
        
        self.creditbox_widthpercent  = creditbox_widthpercent
        self.creditbox_heightpercent = creditbox_heightpercent
        self.creditbox_toppercent    = creditbox_toppercent
        self.creditbox_opacity       = creditbox_opacity
        
        self.button_widthpercent     = button_widthpercent
        self.button_heightpercent    = button_heightpercent
        self.button_elevation        = button_elevation
        self.button_opacity          = button_opacity
        self.button_titlesize        = button_titlesize
        self.button_subtitlesize     = button_subtitlesize
        self.button_radius           = button_radius
        
        self.disclaimer              = disclaimer
        
        self.buttons = []

        

    # Add a button
    def addButton(self, title, subtitle='', tooltip='', image='', onclick=None, argument=None):
        """
        Add a button to the page
        
        Parameters
        ----------
        title : str
            Title of the button
        subtitle : str, optional
            Subtitle of the button (default is '')
        tooltip : str, optional
            Tooltip to show when hovering the button title (default is '')
        image : str, optional
            Image to show on the right side of the button (default is '')
        onclick : function, optional
            Python function to call when the user clicks on the button. The function will receive the argument value as parameter if it is not None, otherwise it will be calle with no parameters. Default is None
        argument : any, optional
            Argument to pass to the onclick python function (default is None)
            
        """
        self.buttons.append({'title':    title,
                             'subtitle': subtitle,
                             'tooltip':  tooltip,
                             'image':    image,
                             'onclick':  onclick,
                             'argument': argument})
        
        
    # Display the page as a fullscreen dialog-box
    def open(self):
        """Open the page"""
        
        # Spacers
        self.spacer = v.Html(tag='div',children=[' '], style_='width: 16px; height: 20px;')

        # Dimension of the main title rectangle
        width = '%fvw'%self.titlebox_widthpercent
        height = '%fvh'%self.titlebox_heightpercent


        
        ######################################################################################################################################################
        # Main titles
        ######################################################################################################################################################
        if self.titleshadow:
            tts = 'text-shadow: -0.06vh -0.06vh 0 %s, 0.06vh -0.06vh 0 %s, -0.06vh 0.06vh 0 %s, 0.06vh 0.06vh 0 %s;'%(self.titleshadow_color,self.titleshadow_color,self.titleshadow_color,self.titleshadow_color)
        else:
            tts = ''
            
        html_main = '''
        <head>
            <meta charset="utf-8" />
            <link href='http://fonts.googleapis.com/css?family=PT+Sans' rel='stylesheet' type='text/css'>
            <style>
                .big {
                    font-family: 'PT Sans', sans-serif;
                    font-size: calc(%f * min(5.0vh, 3.7vw));
                    font-weight: 700;
                    %s
                    text-align: center;
                    line-height: 2.0;
                    color: %s !important; 
                }
                .normal {
                    font-family: 'PT Sans', sans-serif;
                    font-size: calc(%f *min(2.65vh, 2.0vw));
                    font-weight: 700;
                    text-align: center;
                    line-height: 1.0;
                    color: %s !important;
                }
                .responsive4 {
                    width: %fvh;
                    vertical-align: middle;
                    height: auto;
                }
            </style>
        </head>
        <body>
            <div class="big">
                %s
            </div>

            <div class="normal">
                %s
            </div>

            <div class="normal">
                <br/>
                <img src="%s" alt="" class="responsive4">
            </div>
        </body>
        ''' % (self.titlesizepercent*0.01, tts, self.text_color, self.subtitlesizepercent*0.01, self.text_color, self.applogo_widthpercent, self.title, self.subtitle, self.applogo_url)

        cardmain = v.Card(flat=True, children=[HTML(html_main)], style_="background-color: #ffffff00;")

        bcol = '#ffffff%0.2X'%(int(self.titlebox_opacity*255))
        self.hmain = v.Html(tag='div', children=[cardmain], class_="pa-0 ma-0",
                            style_='width: %s; height: %s; border: %dpx solid %s; border-radius: 12px; position: absolute; top: %fvh; left: %fvw; background-color: %s; overflow: hidden;'%(width,height, self.titlebox_border, self.text_color,
                                                                                                                                                                                           self.titlebox_toppercent, (100.0-self.titlebox_widthpercent)/2.0,
                                                                                                                                                                                           bcol))


        ######################################################################################################################################################
        # Credit to vois
        ######################################################################################################################################################
        if self.vois_show:
            html_vois = '''
            <head>
                <meta charset="utf-8" />
                <link href='http://fonts.googleapis.com/css?family=PT+Sans' rel='stylesheet' type='text/css'>
                <style>
                    .normal2 {
                        font-family: 'PT Sans', sans-serif;
                        font-size: min(2.20vh, 1.5vw);
                        font-weight: 700;
                        text-align: center;
                        line-height: 1.0;
                        color: %s;
                    }
                    .responsive2 {
                        width: 10vw;
                        vertical-align: middle;
                        height: auto;
                        background-color: #ffffff%0.2X;
                        padding: 10px;
                    }
                </style>
            </head>
            <body>
                <div class="normal2">
                    <br/>
                    <span style="">created using the: </span>
                    <img src="https://jeodpp.jrc.ec.europa.eu/services/shared/pngs/vois_horizontal_big.png" alt="" class="responsive2">
                </div>
            </body>
            ''' % (self.text_color, int(self.vois_opacity*255))

            cardvois = v.Card(flat=True, children=[HTML(html_vois)], style_="background-color: #ffffff00;")

            self.hvois = v.Html(tag='div', children=[cardvois], class_="pa-0 ma-0",
                                style_='width: 34vw; height: 11vh; border: 0px solid #000000; border-radius: 12px; position: absolute; top: 1vh; left: 65vw; background-color: #ffffff00; overflow: hidden;')
        else:
            self.hvois = ''



        ######################################################################################################################################################
        # Logo and credits
        ######################################################################################################################################################
        html_credits = '''
        <head>
            <meta charset="utf-8" />
            <link href='http://fonts.googleapis.com/css?family=PT+Sans' rel='stylesheet' type='text/css'>
            <style>
                .normal3 {
                    font-family: 'PT Sans', sans-serif;
                    font-size: min(2.0vh, 1.4vw);
                    font-weight: 700;
                    text-align: center;
                    line-height: 1.0;
                    color: %s;
                    overflow: hidden;
                }
                .normal4 {
                    font-family: 'PT Sans', sans-serif;
                    font-size: min(1.3vh, 0.9vw);
                    font-weight: 400;
                    text-align: center;
                    line-height: 1.0;
                    color: %s;
                    overflow: hidden;
                }
                .responsive3 {
                    width: 14vw;
                    vertical-align: middle;
                    height: auto;
                }
            </style>
        </head>
        <body>
            <div class="normal3">
                <img src="%s" alt="" class="responsive3">
            </div>
            <div class="normal3">
                <br/>
                %s
            </div>
            <div class="normal4">
                <br/>
                %s
            </div>
        </body>
        '''%(self.text_color, self.text_color, self.creditslogo_url, self.credits, self.disclaimer)

        cardunit = v.Card(flat=True, children=[HTML(html_credits)], style_="background-color: #ffffff00;", class_="pa-0 ma-0 mt-2")
        bcol = '#ffffff%0.2X'%(int(self.creditbox_opacity*255))
        self.hunit = v.Html(tag='div', children=[cardunit], class_="pa-0 ma-0",
                            style_='width: %fvw; height: %fvh; border: 0px solid #000000; border-radius: 12px; position: absolute; top: %fvh; left: %fvw; background-color: %s; overflow: hidden;'%(self.creditbox_widthpercent, self.creditbox_heightpercent,
                            self.creditbox_toppercent, (100.0-self.creditbox_widthpercent)/2.0, bcol))
        
        nbuttons_per_row = self.buttonbox_widthpercent // self.button_widthpercent
        
        rows = []
        
        children = []
        i = 0
        for b in self.buttons:
            c = buttonCard(elevation=self.button_elevation, width='%fvw'%self.button_widthpercent, 
                           height="%fvh"%self.button_heightpercent, imagesize="%fvh"%self.button_heightpercent,
                           title=b['title'], subtitle=b['subtitle'], tooltiptext=b['tooltip'],
                           color='#ffffff%0.2X'%int(self.button_opacity*255), subtitleweight=400,
                           image=b['image'], on_click=b['onclick'], argument=b['argument'], button_radius=self.button_radius,
                           textcolor=self.text_color, titlesize=self.button_titlesize, subtitlesize=self.button_subtitlesize)
            
            if len(children) > 0:
                children.append(self.spacer)
                children.append(self.spacer)
            children.append(c)
            
            i += 1
            if i >= nbuttons_per_row:
                r = v.Row(justify="center", children=children, class_="pa-0 ma-0 mt-2 mb-6")
                rows.append(r)
                i = 0
                children = []
                
        if len(children) > 0:
            r = v.Row(justify="center", children=children, class_="pa-0 ma-0 mt-2")
            rows.append(r)
            
        hpages = v.Html(tag='div', children=rows, class_="pa-0 ma-0",
                        style_='width: %fvw; height: %fvh; position: absolute; top: %fvh; left: %fvw; background-color: #ffffff00; overflow: hidden;'%(self.buttonbox_widthpercent, self.buttonbox_heightpercent,
                                                                                                                                                       self.buttonbox_toppercent, (100.0-self.buttonbox_widthpercent)/2.0))

        ######################################################################################################################################################
        # Background image
        ######################################################################################################################################################
        if isinstance(self.background_image, int):
            url = 'https://jeodpp.jrc.ec.europa.eu/services/shared/wallpapers/%d.jpg'%self.background_image
        else:
            url = self.background_image

        # See: https://css-tricks.com/perfect-full-page-background-image/
        self.back = v.Html(tag='div', children=[], class_="pa-0 ma-0",
                           style_='''
        width: 100vw; height: 100vh; 
        background-image: url('%s'); no-repeat center center fixed;
        -webkit-background-size: cover;
        -moz-background-size: cover;
        -o-background-size: cover;
        background-size: cover;
        filter: %s;
        '''%(url,self.background_filter))

        display(self.output)
        
        self.dlg = dialogGeneric.dialogGeneric(title='', titleheight='0px', text='', show=True, addclosebuttons=False, 
                                               persistent=RunningInVoila(), no_click_animation=True,    # In JEO-lab the page closes with the "ESC" key!!!
                                               fullscreen=True, content=[self.back, self.hmain, self.hvois, hpages, self.hunit], output=self.output)
        
        

        
    
    # Close the dialog
    def close(self):
        """Close the page"""
        self.page = None
        if not self.dlg is None:
            self.dlg.close()

            
            
    ######################################################################################################################################################
    # Open the main page in preview mode (70% of the fullscreen dimensions!): returns a v.Card widget
    ######################################################################################################################################################
    def preview(self):
        """Open the page in preview mode"""
        
        # Dimension of the main title rectangle
        width = '%f%%'%self.titlebox_widthpercent
        height = '%f%%'%self.titlebox_heightpercent

        if self.titleshadow:
            tts = 'text-shadow: -0.042vh -0.042vh 0 %s, 0.042vh -0.042vh 0 %s, -0.042vh 0.042vh 0 %s, 0.042vh 0.042vh 0 %s;'%(self.titleshadow_color,self.titleshadow_color,self.titleshadow_color,self.titleshadow_color)
        else:
            tts = ''
            
        html_main = '''
        <head>
            <meta charset="utf-8" />
            <link href='http://fonts.googleapis.com/css?family=PT+Sans' rel='stylesheet' type='text/css'>
            <style>
                .big7 {
                    font-family: 'PT Sans', sans-serif;
                    font-size: calc(%f * min(3.5vh, 2.59vw));
                    font-weight: 700;
                    %s
                    text-align: center;
                    line-height: 2.0;
                    color: %s;
                }
                .normal7 {
                    font-family: 'PT Sans', sans-serif;
                    font-size: calc(%f * min(1.855vh, 1.4vw));
                    font-weight: 700;
                    text-align: center;
                    line-height: 1.0;
                    color: %s;
                }
                .responsive7 {
                    width: %fvh;
                    vertical-align: middle;
                    height: auto;
                }
            </style>
        </head>
        <body>
            <div class="big7">
                %s
            </div>

            <div class="normal7">
                %s
            </div>

            <div class="normal7">
                <br/>
                <img src="%s" alt="" class="responsive7">
            </div>
        </body>
        ''' % (self.titlesizepercent*0.01, tts, self.text_color, self.subtitlesizepercent*0.01, self.text_color, self.applogo_widthpercent, self.title, self.subtitle, self.applogo_url)

        cardmain = v.Card(flat=True, children=[HTML(html_main)], style_="background-color: #ffffff00;")

        bcol = '#ffffff%0.2X'%(int(self.titlebox_opacity*255))
        hmain = v.Html(tag='div', children=[cardmain], class_="pa-0 ma-0",
                       style_='width: %s; height: %s; border: %dpx solid %s; border-radius: 12px; position: absolute; top: %f%%; left: %f%%; background-color: %s; overflow: hidden;'%(width,height, self.titlebox_border, self.text_color,
                                                                                                                                                                                       self.titlebox_toppercent, (100.0-self.titlebox_widthpercent)/2.0,
                                                                                                                                                                                       bcol))
        
        
        ######################################################################################################################################################
        # Credit to vois
        ######################################################################################################################################################
        if self.vois_show:
            html_vois = '''
            <head>
                <meta charset="utf-8" />
                <link href='http://fonts.googleapis.com/css?family=PT+Sans' rel='stylesheet' type='text/css'>
                <style>
                    .normal27 {
                        font-family: 'PT Sans', sans-serif;
                        font-size: min(1.54vh, 1.05vw);
                        font-weight: 700;
                        text-align: center;
                        line-height: 1.0;
                        color: %s;
                    }
                    .responsive27 {
                        width: 7vw;
                        vertical-align: middle;
                        height: auto;
                        background-color: #ffffff%0.2X;
                        padding: 7px;
                    }
                </style>
            </head>
            <body>
                <div class="normal27">
                    <br/>
                    <span style="">created using the: </span>
                    <img src="https://jeodpp.jrc.ec.europa.eu/services/shared/pngs/vois_horizontal_big.png" alt="" class="responsive27">
                </div>
            </body>
            ''' % (self.text_color, int(self.vois_opacity*255))

            cardvois = v.Card(flat=True, children=[HTML(html_vois)], style_="background-color: #ffffff00;")

            hvois = v.Html(tag='div', children=[cardvois], class_="pa-0 ma-0",
                                style_='width: 34%; height: 11%; border: 0px solid #000000; border-radius: 12px; position: absolute; top: 1%; left: 65%; background-color: #ffffff00; overflow: hidden;')
        else:
            hvois = ''



        ######################################################################################################################################################
        # Logo and credits
        ######################################################################################################################################################
        html_credits = '''
        <head>
            <meta charset="utf-8" />
            <link href='http://fonts.googleapis.com/css?family=PT+Sans' rel='stylesheet' type='text/css'>
            <style>
                .normal37 {
                    font-family: 'PT Sans', sans-serif;
                    font-size: min(1.4vh, 0.98vw);
                    font-weight: 700;
                    text-align: center;
                    line-height: 1.0;
                    color: %s;
                    overflow: hidden;
                }
                .normal47 {
                    font-family: 'PT Sans', sans-serif;
                    font-size: min(0.91vh, 0.63vw);
                    font-weight: 400;
                    text-align: center;
                    line-height: 1.0;
                    color: %s;
                    overflow: hidden;
                }
                .responsive37 {
                    width: 9.8vw;
                    vertical-align: middle;
                    height: auto;
                }
            </style>
        </head>
        <body>
            <div class="normal37">
                <img src="%s" alt="" class="responsive37">
            </div>
            <div class="normal37">
                <br/>
                %s
            </div>
            <div class="normal47">
                <br/>
                %s
            </div>
        </body>
        '''%(self.text_color, self.text_color, self.creditslogo_url, self.credits, self.disclaimer)

        cardunit = v.Card(flat=True, children=[HTML(html_credits)], style_="background-color: #ffffff00;", class_="pa-0 ma-0 mt-2")
        bcol = '#ffffff%0.2X'%(int(self.creditbox_opacity*255))
        hunit = v.Html(tag='div', children=[cardunit], class_="pa-0 ma-0",
                       style_='width: %f%%; height: %f%%; border: 0px solid #000000; border-radius: 12px; position: absolute; top: %f%%; left: %f%%; background-color: %s; overflow: hidden;'%(self.creditbox_widthpercent, self.creditbox_heightpercent,
                       self.creditbox_toppercent, (100.0-self.creditbox_widthpercent)/2.0, bcol))
        
        
        nbuttons_per_row = self.buttonbox_widthpercent // self.button_widthpercent
        
        rows = []
        
        children = []
        i = 0
        for b in self.buttons:
            c = buttonCard(elevation=self.button_elevation, width='%fvw'%(0.7*self.button_widthpercent), 
                           height="%fvh"%(0.7*self.button_heightpercent), imagesize="%fvh"%(0.7*self.button_heightpercent),
                           title=b['title'], subtitle=b['subtitle'], tooltiptext=b['tooltip'],
                           color='#ffffff%0.2X'%int(self.button_opacity*255), subtitleweight=400,
                           image=b['image'], on_click=b['onclick'], argument=b['argument'], button_radius=self.button_radius,
                           textcolor=self.text_color, titlesize='calc(0.7 * %s)'%self.button_titlesize, subtitlesize='calc(0.7 * %s)'%self.button_subtitlesize)
            
            if len(children) > 0:
                children.append(self.spacer)
                children.append(self.spacer)
            children.append(c)
            
            i += 1
            if i >= nbuttons_per_row:
                r = v.Row(justify="center", children=children, class_="pa-0 ma-0 mt-2 mb-6")
                rows.append(r)
                i = 0
                children = []
                
        if len(children) > 0:
            r = v.Row(justify="center", children=children, class_="pa-0 ma-0 mt-2")
            rows.append(r)
            
        hpages = v.Html(tag='div', children=rows, class_="pa-0 ma-0",
                        style_='width: %f%%; height: %f%%; position: absolute; top: %f%%; left: %f%%; background-color: #ffffff00; overflow: hidden;'%(self.buttonbox_widthpercent, self.buttonbox_heightpercent,
                                                                                                                                                       self.buttonbox_toppercent, (100.0-self.buttonbox_widthpercent)/2.0))
        
        # Background image
        if isinstance(self.background_image, int):
            url = 'https://jeodpp.jrc.ec.europa.eu/services/shared/wallpapers/%d.jpg'%self.background_image
        else:
            url = self.background_image

        # See: https://css-tricks.com/perfect-full-page-background-image/
        back = v.Html(tag='div', children=[], class_="pa-0 ma-0",style_='''
        width: 70vw; height: 70vh; 
        background-image: url('%s'); no-repeat center center fixed;
        -webkit-background-size: cover;
        -moz-background-size: cover;
        -o-background-size: cover;
        background-size: cover;
        filter: %s;
        '''%(url,self.background_filter))

        return v.Card(children=[back, hmain, hvois, hpages, hunit], width='70vw', style_='border-radius: 0px;')
    
    
        
    ######################################################################################################################################################
    # Save the configuration into a JSON file
    ######################################################################################################################################################
    def toJson(self):

        return {
            'title': self.title,
            'subtitle': self.subtitle,
            'titlesizepercent': self.titlesizepercent,
            'subtitlesizepercent': self.subtitlesizepercent,
            'applogo_widthpercent': self.applogo_widthpercent,
            'credits': self.credits,
            'text_color': self.text_color,
            'background_image': self.background_image,
            'background_filter': self.background_filter,
            'vois_show': self.vois_show,
            'vois_opacity': self.vois_opacity,

            'titlebox_widthpercent': self.titlebox_widthpercent,
            'titlebox_heightpercent': self.titlebox_heightpercent,
            'titlebox_toppercent': self.titlebox_toppercent,
            'titlebox_opacity': self.titlebox_opacity,
            'titlebox_border': self.titlebox_border,

            'titleshadow': self.titleshadow,
            'titleshadow_color': self.titleshadow_color,

            'buttonbox_widthpercent': self.buttonbox_widthpercent,
            'buttonbox_heightpercent': self.buttonbox_heightpercent,
            'buttonbox_toppercent': self.buttonbox_toppercent,

            'creditbox_widthpercent': self.creditbox_widthpercent,
            'creditbox_heightpercent': self.creditbox_heightpercent,
            'creditbox_toppercent': self.creditbox_toppercent,
            'creditbox_opacity': self.creditbox_opacity,

            'button_widthpercent': self.button_widthpercent,
            'button_heightpercent': self.button_heightpercent,
            'button_elevation': self.button_elevation,
            'button_opacity': self.button_opacity,
            'button_titlesize': self.button_titlesize,
            'button_subtitlesize': self.button_subtitlesize,
            'button_radius': self.button_radius,

            'disclaimer': self.disclaimer,

            'buttons': self.buttons,

            'applogo_url': self.applogo_url,
            'creditslogo_url': self.creditslogo_url,
        }

    
    ######################################################################################################################################################
    # Read the configuration into a JSON file
    ######################################################################################################################################################
    def fromJson(self, j):

            self.title = j['title']
            self.subtitle = j['subtitle']
            self.titlesizepercent = j['titlesizepercent']
            self.subtitlesizepercent = j['subtitlesizepercent']
            self.applogo_url = j['applogo_url']
            self.applogo_widthpercent = j['applogo_widthpercent']
            self.credits = j['credits']
            self.creditslogo_url = j['creditslogo_url']
            self.text_color = j['text_color']
            self.background_image = j['background_image']
            self.background_filter = j['background_filter']
            self.vois_show = j['vois_show']
            self.vois_opacity = j['vois_opacity']

            self.titlebox_widthpercent = j['titlebox_widthpercent']
            self.titlebox_heightpercent = j['titlebox_heightpercent']
            self.titlebox_toppercent = j['titlebox_toppercent']
            self.titlebox_opacity = j['titlebox_opacity']
            self.titlebox_border = j['titlebox_border']

            self.titleshadow = j['titleshadow']
            self.titleshadow_color = j['titleshadow_color']

            self.buttonbox_widthpercent = j['buttonbox_widthpercent']
            self.buttonbox_heightpercent = j['buttonbox_heightpercent']
            self.buttonbox_toppercent = j['buttonbox_toppercent']

            self.creditbox_widthpercent = j['creditbox_widthpercent']
            self.creditbox_heightpercent = j['creditbox_heightpercent']
            self.creditbox_toppercent = j['creditbox_toppercent']
            self.creditbox_opacity = j['creditbox_opacity']

            self.button_widthpercent = j['button_widthpercent']
            self.button_heightpercent = j['button_heightpercent']
            self.button_elevation = j['button_elevation']
            self.button_opacity = j['button_opacity']
            self.button_titlesize = j['button_titlesize']
            self.button_subtitlesize = j['button_subtitlesize']
            self.button_radius = j['button_radius']

            self.disclaimer = j['disclaimer']

            self.buttons = j['buttons']
    


"""Example of image display to use inside a Content"""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2024
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.

# Imports
from ipywidgets import widgets, HTML, Layout
import ipyvuetify as v
import pandas as pd
import random

# Vois imports
from vois.vuetify import settings, Button, UploadImage
from vois.templates import PageConfigurator


#####################################################################################################################################################
# Example of image display to use inside a Content
#####################################################################################################################################################
class Image():
    
    def __init__(self,
                 output,
                 width='30vw',        # Dimensions of the image
                 height='40vh',
                 color_first=None,    # Main color
                 color_second=None,   # Secondary color
                 dark=None,           # Dark flag
                 imageurl='',         # Image to display
                 **kwargs):
        
        self.output = output
        
        self._width    = width
        self._height   = height
        self._imageurl = imageurl
        
        # Colors of the configuration widgets
        self._color_first = color_first
        if self._color_first is None:
            self._color_first = settings.color_first
        
        self._color_second = color_second
        if self._color_second is None:
            self._color_second = settings.color_second
            
        self._dark = dark
        if self._dark is None:
            self._dark = settings.dark_mode

        # Create the card that will contain the image
        self.card = v.Card(flat=True, tile=True, width=self._width, height=self._height,
                           style_='overflow: hidden;', class_='d-flex align-center justify-center')  # The content of the card is centered horizontally and vertically
        
        self.img = v.Img(src=self._imageurl, contain=True, width=self._width, height=self._height, position='center center')
        
        self.tf = None
        self.b  = None
        
        self.card.children = [self.img]
        
        
    # Draw th widget
    def draw(self):
        return self.card
    
    # Called when an image is selected
    def onImageSelected(self, imageurl):
        self.imageurl = imageurl
        
        
    # Selection of an image
    def onSelectImage(self, *args):
        upload = UploadImage.UploadImage(self.output, width=620)
        upload.title   = 'Select image to display'
        upload.onOK    = self.onImageSelected
        upload.color   = self._color_first
        upload.u.color = self._color_first
        upload.dark    = self._dark
        upload.show()
        
    # Changed image URL in the self.tf TextField
    def onChangedURL(self, *args):
        if self.tf.v_model is None:
            url = ''
        else:
            url = self.tf.v_model
        self.imageurl = url

        
    # Configure the SVG drawing
    def configure(self):
        
        self.tf = v.TextField(label='Image URL:', autofocus=False, v_model=None, dense=False, color=self._color_first, clearable=True, class_="pa-0 ma-0 mt-3 mb-1")
        self.tf.on_event('change', self.onChangedURL)
        
        self.b  = Button('Upload image from local computer', color_selected=self._color_first, dark=self._dark, 
                         text_weight=450, on_click=self.onSelectImage, width=300, height=40,
                         tooltip='Click to select an image to upload from you local computer', selected=True,
                         rounded=False)
        
        return v.Card(flat=True, class_='pa-2 ma-0', children=[widgets.VBox([PageConfigurator.label('Image', color='black'), self.tf, self.b])])


    @property
    def content(self):
        return 'Image'
        
    @content.setter
    def content(self, c):
        pass
    
    
    @property
    def width(self):
        return self._width
        
    @width.setter
    def width(self, w):
        self._width = w
        self.card.width = w
        self.img.width = w

        
    @property
    def height(self):
        return self._height
        
    @height.setter
    def height(self, h):
        self._height = h
        self.card.height = h
        self.img.height = h

        
    @property
    def imageurl(self):
        return self._imageurl
        
    @imageurl.setter
    def imageurl(self, url):
        self._imageurl = url
        self.img.src = self._imageurl
        
        
    @property
    def color_first(self):
        return self._color_first
        
    @color_first.setter
    def color_first(self, color):
        self._color_first = color

        if self.b is not None:
            self.b.color_selected = self._color_first
            
        if self.tf is not None:
            self.tf.color = self._color_first

    @property
    def color_second(self):
        return self._color_second
        
    @color_second.setter
    def color_second(self, color):
        self._color_second = color
    
    
    @property
    def dark(self):
        return self._dark
        
    @dark.setter
    def dark(self, flag):
        self._dark = flag
        
        
    @property
    def state(self):
        return {x: getattr(self, x) for x in ['content',
                                              #'width',      # Will inherit from content!!!
                                              #'height',
                                              'imageurl',
                                              'color_first',
                                              'color_second',
                                              'dark'
                                             ]}
        
    @state.setter
    def state(self, statusdict):
        for key, value in statusdict.items():
            setattr(self, key, value)


"""Example of Plotly Chart that is centered in a v.Card"""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2024
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.

# Imports
from ipywidgets import widgets, Layout
import ipyvuetify as v
import plotly.express as px

# Vois imports
from vois.vuetify import settings, sliderFloat
from vois.templates import PageConfigurator


#####################################################################################################################################################
# Example of Plotly Chart that is centered in a v.Card
#####################################################################################################################################################
class PlotlyChart(v.Card):
    
    def __init__(self,
                 width='99%',             # Dimensions of the overall card displaying the chart
                 height='400px',
                 chart_width=500,         # Chart width in pixels
                 chart_height=250,        # Chart height in pixels
                 color_first=None,        # Main color
                 color_second=None,       # Secondary color
                 dark=None,               # Dark flag
                 **kwargs
                ):
        
        super().__init__(flat=True, tile=True, width=width, height=height,
                         style_='overflow: hidden;', class_='d-flex align-center justify-center',
                         **kwargs)
        
        # Dimensions in pixels of the chart
        self._chart_width  = chart_width
        self._chart_height = chart_height
        
        # Colors of the configuration widgets
        self._color_first = color_first
        if self._color_first is None:
            self._color_first = settings.color_first
        
        self._color_second = color_second
        if self._color_second is None:
            self._color_second = settings.color_second
            
        self._dark = dark
        if self._dark is None:
            self._dark = settings.dark_mode
            
        self.configure_card = None
        self.sw = self.sh = None
        self.update_properties = True

        # Ouput and clipping card
        self.output = widgets.Output(layout=Layout(width='%dpx'%self._chart_width, height='%dpx'%(50 + self._chart_height)))
        self.clip = v.Card(flat=True, tile=True, width='%dpx'%self._chart_width, height='%dpx'%self._chart_height, children=[self.output], style_='overflow: hidden;')
        
        # Sample chart
        df = px.data.iris()
        self._fig = px.scatter(df, x="sepal_width", y="sepal_length", color="species")
        self._fig.update_layout(title='Sample chart', template='plotly_white', autosize=False, width=self._chart_width, height=self._chart_height, margin=dict(l=0, r=0, b=0, t=30))
        
        # Display figure in the output widget
        with self.output:
            display(self._fig.show())
            
        # Set the clipped Card as the children of the overall Card
        self.children = [self.clip]
        

    # Fake "draw" method
    def draw(self):
        return self
        

    # Update when chart_width or chart_height properties are set
    def update(self):
        self.output.layout.width  = '%dpx'%self._chart_width
        self.output.layout.height = '%dpx'%(50 + self._chart_height)
        self.clip.width  = '%dpx'%self._chart_width
        self.clip.height = '%dpx'%self._chart_height

        self.output.clear_output(wait=True)
        with self.output:
            display(self._fig.show())
        
        
    @property
    def content(self):
        return 'Plotly Chart'
        
    @content.setter
    def content(self, c):
        pass
    
    
    @property
    def chart_width(self):
        return self._chart_width
        
    @chart_width.setter
    def chart_width(self, w):
        self._chart_width = int(w)
        self._fig.update_layout(width=self._chart_width)
        self.update()

        if self.sw is not None:
            self.update_properties = False
            self.sw.value = self._chart_width
            self.update_properties = True
        
        
    @property
    def chart_height(self):
        return self._chart_height
        
    @chart_height.setter
    def chart_height(self, h):
        self._chart_height = int(h)
        self._fig.update_layout(height=self._chart_height)
        self.update()
        
        if self.sh is not None:
            self.update_properties = False
            self.sh.value = self._chart_height
            self.update_properties = True
        
        
        
    @property
    def color_first(self):
        return self._color_first
        
    @color_first.setter
    def color_first(self, color):
        self._color_first = color

        if self.sw is not None:
            self.sw.slider.color = color
            
        if self.sh is not None:
            self.sh.slider.color = color


    @property
    def color_second(self):
        return self._color_second
        
    @color_second.setter
    def color_second(self, color):
        self._color_second = color


    @property
    def dark(self):
        return self._dark
        
    @dark.setter
    def dark(self, flag):
        self._dark = flag

            
    @property
    def fig(self):
        return self._fig
        
    @fig.setter
    def fig(self, f):
        self._fig = f
        self._fig.update_layout(width=self._chart_width)
        self._fig.update_layout(height=self._chart_height)
        
        self.output.clear_output(wait=True)
        with self.output:
            display(self._fig.show())
        
        
        
        
    def changeW(self, w):
        if self.update_properties:
            self.chart_width = w
        
    def changeH(self, h):
        if self.update_properties:
            self.chart_height = h
        
        
    # Create a card instance to configure the chart dimensions in pixels
    def configure(self):
        if self.configure_card is None:
            self.configure_card = v.Card(flat=True, class_='pa-2 ma-0')
            
            self.sw = sliderFloat.sliderFloat(self._chart_width, text='Chart width in pixels:', minvalue=20.0, maxvalue=1000.0, maxint=980, showpercentage=False, decimals=0,
                                                   labelwidth=96, sliderwidth=150, resetbutton=True, showtooltip=True, onchange=self.changeW)
            
            self.sh = sliderFloat.sliderFloat(self._chart_height, text='Chart height in pixels:', minvalue=20.0, maxvalue=800.0, maxint=780, showpercentage=False, decimals=0,
                                                   labelwidth=96, sliderwidth=150, resetbutton=True, showtooltip=True, onchange=self.changeH)
            
            self.configure_card.children = [widgets.VBox([PageConfigurator.label('Plotly chart', color='black'), self.sw.draw(), self.sh.draw()])]
            
        return self.configure_card

    
    @property
    def state(self):
        return {x: getattr(self, x) for x in ['content',
                                              'chart_width',
                                              'chart_height',
                                              'color_first',
                                              'color_second',
                                              'dark'
                                             ]}
        
    @state.setter
    def state(self, statusdict):
        for key, value in statusdict.items():
            setattr(self, key, value)
    


"""Main content of the template1/2/3panels classes"""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2024
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.

# Imports
from ipywidgets import widgets, Layout
import ipyvuetify as v

# Vois imports
from vois.vuetify import settings, toggle, selectSingle, dialogWait, sliderFloat, switch
from vois.geo import Map, mapUtils
from vois.templates import PlotlyChart, SVGdrawing, Image, PageConfigurator


#####################################################################################################################################################
# Main content of a template1/2/3panels class
#####################################################################################################################################################
class Content(v.Card):

    # Initialization
    def __init__(self,
                 output,
                 width='50vw',            # Overall width
                 height='50vh',           # Overall height
                 splitmode=0,             # 0=single content,  1=two contents splitted vertically,   2=two contents splitted horizontally,   3=three contents,   4=four contents
                 leftwidthperc=50,        # width in percentage of left column
                 topheightperc=50,        # height in percentage of top row
                 borderwidth=2,           # Number of pixels for the border
                 linkedmaps=False,        # If True all the maps bounds are linked
                 color_first=None,        # Main color
                 color_second=None,       # Secondary color
                 dark=None,               # Dark flag
                 onStateChanged=None,     # Called just after the state property is changes
                 **kwargs):
        
        super().__init__(**kwargs)
        
        self.output         = output
        self._width         = width
        self._height        = height
        self._splitmode     = splitmode
        self._leftwidthperc = leftwidthperc
        self._topheightperc = topheightperc
        self._borderwidth   = borderwidth
        self._linkedmaps    = linkedmaps
        
        self.onStateChanged = onStateChanged
        
        self.debug = widgets.Output()
        
        # Dimensioning
        self.labelwidth   = 96
        self.togglewidth  = 46
        self.paddingrow   = 1
        
        self.spacerX = v.Html(tag='div', style_='width: 10px; height:  0px;')
        self.spacerY = v.Html(tag='div', style_='width:  0px; height: 10px;')
        self.spacer  = v.Html(tag='div', style_='width: 10px; height: 10px;')
        
        # Flag to update controls when loading the state
        self.updateWidgetsOnLoadState = False
        
        self.reset_state = {
 'splitmode': 0,
 'leftwidthperc': 50,
 'topheightperc': 50,
 'borderwidth': 2,
 'linkedmaps': False,
 'color_first': '#0d856d',
 'color_second': '#a0dcd0',
 'dark': True,
 'content1': {
    'content': 'Map',
    'show_fullscreen': True,
    'show_coordinates': True,
    'show_search': True,
    'show_scale': True,
    'show_basemaps': True,
    'show_overview': False,
    'color_first': '#0d856d',
    'color_second': '#a0dcd0',
    'dark': True,
    'basemapindex': 0,
    'center': [50, 12],
    'zoom': 5.0
    }
}
        
        # List of all the Map.Map instances
        self.maps = []
        
        # Colors of the configuration widgets
        self._color_first = color_first
        if self._color_first is None:
            self._color_first = settings.color_first
        
        self._color_second = color_second
        if self._color_second is None:
            self._color_second = settings.color_second
            
        self._dark = dark
        if self._dark is None:
            self._dark = settings.dark_mode

        # Main card
        self.card = v.Card(flat=True, color='#ffffff', tile=True,
                           width=self._width,   min_width=self._width,   max_width=self._width,
                           height=self._height, min_height=self._height, max_height=self._height)
        
        self.card1 = self.card2 = self.card3 = self.card4 = None
        self.card1children = self.card2children = self.card3children = self.card4children = None    # Widgets to display in the component cards
        
        m = Map.Map()
        self.set1(m)
        m.observe(self.onMapBoundsChanged, 'bounds')
        
        self.children = [self.card]
        
        
        # Widgets for confugure GUI
        self.toggle_splitmode = toggle.toggle(self.splitmode, ['', '', '', '', ''], dark=self._dark, icons=['mdi-border-all-variant', 'mdi-dock-right', 'mdi-view-agenda-outline', 'mdi-view-compact-outline', 'mdi-border-all'], outlined=False,
                                              tooltips=['Single area', 'Two areas splitted horizontally', 'Two areas splitted vertically', 'Three areas', 'Four areas'],
                                              onchange=self.splitmodeChange, row=True, width=self.togglewidth, height=30, justify='start', paddingrow=self.paddingrow, tile=True)

        contents = ['None', 'Map', 'Plotly Chart', 'SVG Drawing', 'Image']
        self.select1 = selectSingle.selectSingle('Content for area 1:', contents, selection='Map',  clearable=False, width=200, onchange=self.onselect1Change)
        self.select2 = selectSingle.selectSingle('Content for area 2:', contents, selection='None', clearable=False, width=200, onchange=self.onselect2Change)
        self.select2.disabled = True
        self.select3 = selectSingle.selectSingle('Content for area 3:', contents, selection='None', clearable=False, width=200, onchange=self.onselect3Change)
        self.select3.disabled = True
        self.select4 = selectSingle.selectSingle('Content for area 4:', contents, selection='None', clearable=False, width=200, onchange=self.onselect4Change)
        self.select4.disabled = True
        
        self.sliderleftwidth = sliderFloat.sliderFloat(50, text='Left column percent:', showpercentage=True, decimals=0, minvalue=0.0, maxvalue=100.0, maxint=100,
                                                       labelwidth=self.labelwidth-10, sliderwidth=150, resetbutton=True, showtooltip=True, onchange=self.leftwidthChange)
        self.sliderleftwidth.slider.disabled = True
        
        self.slidertopheight = sliderFloat.sliderFloat(50, text='Top row percent:', showpercentage=True, decimals=0, minvalue=0.0, maxvalue=100.0, maxint=100,
                                                       labelwidth=self.labelwidth-10, sliderwidth=150, resetbutton=True, showtooltip=True, onchange=self.topheightChange)
        self.slidertopheight.slider.disabled = True

        self.mapslinked = switch.switch(self._linkedmaps, 'Maps linked', inset=True, dense=True, onchange=self.linkedChange)
        self.mapslinked.disabled = True
        self.propagateBounds = True
        self.showWaitDialogOnUpdate = True

        self.selectborder = selectSingle.selectSingle('Border in pixel:', ['0', '1', '2', '3'], selection=str(self._borderwidth),  clearable=False, width=100, onchange=self.onselectBorder)

            
        self.toggle_configure = toggle.toggle(0, ['', '', '', ''], dark=self._dark, icons=['mdi-numeric-1-box-outline', 'mdi-numeric-2-box-outline', 'mdi-numeric-3-box-outline', 'mdi-numeric-4-box-outline'], outlined=False,
                                              tooltips=['Configure area 1', 'Configure area 2', 'Configure area 3', 'Configure area 4'],
                                              onchange=self.configureChange, row=True, width=self.togglewidth, height=30, justify='start', paddingrow=self.paddingrow, tile=True)
        self.toggle_configure.buttons[1].disabled = True
        self.toggle_configure.buttons[2].disabled = True
        self.toggle_configure.buttons[3].disabled = True
        
        
        self.card_configure = v.Card(flat=True)
        
        self.splitmodeChange(self.splitmode)
        self.configureChange(0)
        self.update()
        
        self.createAccessStrings()

        
    # Create strings for easy access to content
    def createAccessStrings(self):
        dnames = {
            Map.Map: 'map',
            PlotlyChart.PlotlyChart: 'chart',
            SVGdrawing.SVGdrawing:   'svg',
            Image.Image:             'image'
        }
        self.access1 = self.access2 = self.access3 = self.access4 = ''
        if self.card1children is not None: self.access1 = "self.%s1 = self.content.card1children"%dnames[type(self.card1children)]
        if self.card2children is not None: self.access2 = "self.%s2 = self.content.card2children"%dnames[type(self.card2children)]
        if self.card3children is not None: self.access3 = "self.%s3 = self.content.card3children"%dnames[type(self.card3children)]
        if self.card4children is not None: self.access4 = "self.%s4 = self.content.card4children"%dnames[type(self.card4children)]

        
    # Selection of the area to configure
    def configureChange(self, index):
        area_configure = ''
        if index == 0:
            if self.card1children is not None:
                area_configure = self.card1children.configure()
        elif index == 1:
            if self.card2children is not None:
                area_configure = self.card2children.configure()
        elif index == 2:
            if self.card3children is not None:
                area_configure = self.card3children.configure()
        elif index == 3:
            if self.card4children is not None:
                area_configure = self.card4children.configure()
        
        self.card_configure.children = [area_configure]
        
        
    # GUI interface for content selection
    def configure(self):
        return v.Card(flat=True, children=[widgets.VBox([
            self.spacerY,
            widgets.HBox([PageConfigurator.label('Split mode:', color='black', width=self.labelwidth), self.toggle_splitmode.draw()]),
            self.spacerY,
            self.sliderleftwidth.draw(),
            self.slidertopheight.draw(),
            self.spacerY,
            self.spacerY,
            widgets.HBox([self.select1.draw(), self.spacerX, self.mapslinked.draw()]),
            widgets.HBox([self.select2.draw(), self.spacerX, self.spacerX, self.selectborder.draw()]),
            self.select3.draw(),
            self.select4.draw(),
            self.spacerY,
            widgets.HBox([PageConfigurator.label('Content: ', color='black', width=self.labelwidth-40), self.toggle_configure.draw()]),
            self.spacerY,
            self.card_configure,
        ])])
        
        
    # Reset the state to the initial one
    def reset(self):
        self.state = self.reset_state
    
    
    # Selecton of the splitmode
    def splitmodeChange(self, index):
        self.splitmode = index
    

    # Change of an area content
    def changeAreaContent(self, setfunction, contentname):
        
        if self.showWaitDialogOnUpdate:
            dlg = dialogWait.dialogWait(text='Updating content...', output=self.output, color=self._color_first, dark=self._dark)
        
        if contentname == 'Map':
            m = Map.Map(color_first=self._color_first, color_second=self._color_second, dark=self._dark)
            setfunction(m)
            m.observe(self.onMapBoundsChanged, 'bounds')
        elif contentname == 'Plotly Chart':
            setfunction(PlotlyChart.PlotlyChart(color_first=self._color_first, color_second=self._color_second, dark=self._dark))
        elif contentname == 'SVG Drawing':
            setfunction(SVGdrawing.SVGdrawing(color_first=self._color_first, color_second=self._color_second, dark=self._dark))
        elif contentname == 'Image':
            setfunction(Image.Image(self.output, color_first=self._color_first, color_second=self._color_second, dark=self._dark))
        else:
            setfunction(None)
            
        # Count the number of Maps and if more than 1 enable the linked switch
        self.maps = []
        if isinstance(self.card1children, Map.Map): self.maps.append(self.card1children)
        if isinstance(self.card2children, Map.Map): self.maps.append(self.card2children)
        if isinstance(self.card3children, Map.Map): self.maps.append(self.card3children)
        if isinstance(self.card4children, Map.Map): self.maps.append(self.card4children)
        
        if len(self.maps) < 2:
            self.mapslinked.disabled = True
        else:
            self.mapslinked.disabled = False
            
        self.configureChange(self.toggle_configure.value)

        self.createAccessStrings()
        
        if self.showWaitDialogOnUpdate:
            dlg.close()
                       

            
    # Management of the linked state among the maps
    def onMapBoundsChanged(self, change):
        if self.propagateBounds:
            if len(self.maps) > 1:
                if self._linkedmaps:
                    inputMap = change['owner']
                    for m in self.maps:
                        if not m is inputMap:
                            self.propagateBounds = False
                            m.center = inputMap.center
                            m.zoom   = inputMap.zoom
                            self.propagateBounds = True
    
    
    # Select* change
    def onselect1Change(self):
        self.changeAreaContent(self.set1, self.select1.value)
        
    def onselect2Change(self):
        self.changeAreaContent(self.set2, self.select2.value)
        
    def onselect3Change(self):
        self.changeAreaContent(self.set3, self.select3.value)
        
    def onselect4Change(self):
        self.changeAreaContent(self.set4, self.select4.value)
                               
    
    # Change of left width
    def leftwidthChange(self, value):
        self.leftwidthperc = int(value)
    
    
    # Change of the top height
    def topheightChange(self, value):
        self.topheightperc = int(value)
    
    
    # Set the linked flag
    def linkedChange(self, flag):
        self._linkedmaps = self.mapslinked.value
        if self._linkedmaps:
            if len(self.maps) > 1:
                self.onMapBoundsChanged({'owner': self.maps[0]})
    
    
    # Selection of a border
    def onselectBorder(self):
        self.propagateBounds = False
        self.borderwidth = int(self.selectborder.value)
        self.propagateBounds = True

    
    # Returns the vuetify object to display (the v.Card)
    def draw(self):
        return self
    

    # Update all colors
    def updateColors(self):
        if self.card1children is not None: self.card1children.color_first = self._color_first
        if self.card2children is not None: self.card2children.color_first = self._color_first
        if self.card3children is not None: self.card3children.color_first = self._color_first
        if self.card4children is not None: self.card4children.color_first = self._color_first
        
        if self.card1children is not None: self.card1children.color_second = self._color_second
        if self.card2children is not None: self.card2children.color_second = self._color_second
        if self.card3children is not None: self.card3children.color_second = self._color_second
        if self.card4children is not None: self.card4children.color_second = self._color_second
        
        if self.card1children is not None: self.card1children.dark = self._dark
        if self.card2children is not None: self.card2children.dark = self._dark
        if self.card3children is not None: self.card3children.dark = self._dark
        if self.card4children is not None: self.card4children.dark = self._dark
        
        
    # Update the content when splitmode is changed
    def update(self):

        wl = 'calc(%s * %f)'%(self._width, self._leftwidthperc/100)
        wr = 'calc(%s * %f)'%(self._width, (100 - self._leftwidthperc)/100)
        ht = 'calc(%s * %f)'%(self._height, self._topheightperc/100)
        hb = 'calc(%s * %f)'%(self._height, (100 - self._topheightperc)/100)
        
        # Single content
        if self._splitmode == 0:
            self.card1 = v.Card(flag=True, tile=True, style_='overflow: hidden;',
                                width=self._width,   min_width=self._width,   max_width=self._width,
                                height=self._height, min_height=self._height, max_height=self._height)
            self.card2 = None
            self.card3 = None
            self.card4 = None
            
            self.card.children = [self.card1]
            self.toggle_configure.value = 0
            self.toggle_configure.buttons[1].disabled = True
            self.toggle_configure.buttons[2].disabled = True
            self.toggle_configure.buttons[3].disabled = True
        
        # 2 horizontal contents
        elif self._splitmode == 1:
            self.card1 = v.Card(flag=True, tile=True, outlined=True, style_='border: 0px solid red; border-right: %dpx solid %s; overflow: hidden;'%(self._borderwidth,self._color_first),
                                width=wl, min_width=wl, max_width=wl,
                                height=self._height, min_height=self._height, max_height=self._height)
            self.card2 = v.Card(flag=True, tile=True, 
                                width=wr, min_width=wr, max_width=wr,
                                height=self._height, min_height=self._height, max_height=self._height)
    
            self.card3 = None
            self.card4 = None
            
            self.card.children = [widgets.HBox([self.card1, self.card2])]
            if self.toggle_configure.value > 1: self.toggle_configure.value = 0
            self.toggle_configure.buttons[1].disabled = False
            self.toggle_configure.buttons[2].disabled = True
            self.toggle_configure.buttons[3].disabled = True
            
            
        # 2 vertical contents
        elif self._splitmode == 2:
            self.card1 = v.Card(flag=True, tile=True, outlined=True, style_='border: 0px solid red; border-bottom: %dpx solid %s; overflow: hidden;'%(self._borderwidth,self._color_first),
                                width=self._width, min_width=self._width, max_width=self._width,
                                height=ht, min_height=ht, max_height=ht)
            self.card2 = v.Card(flag=True, tile=True, 
                                width=self._width, min_width=self._width, max_width=self._width,
                                height=hb, min_height=hb, max_height=hb)
    
            self.card3 = None
            self.card4 = None
            
            self.card.children = [widgets.VBox([self.card1, self.card2])]
            if self.toggle_configure.value > 1: self.toggle_configure.value = 0
            self.toggle_configure.buttons[1].disabled = False
            self.toggle_configure.buttons[2].disabled = True
            self.toggle_configure.buttons[3].disabled = True

        # 2 vertical contents + 1 on the right at full height
        elif self._splitmode == 3:
            self.card1 = v.Card(flag=True, tile=True, outlined=True, style_='border: 0px solid red; border-right: %dpx solid %s; border-bottom: %dpx solid %s; overflow: hidden;'%(self._borderwidth, self._color_first,
                                                                                                                                                                                   self._borderwidth, self._color_first),
                                width=wl, min_width=wl, max_width=wl,
                                height=ht, min_height=ht, max_height=ht)
            self.card2 = v.Card(flag=True, tile=True, outlined=True, style_='border: 0px solid red; border-right: %dpx solid %s; overflow: hidden;'%(self._borderwidth,self._color_first),
                                width=wl, min_width=wl, max_width=wl,
                                height=hb, min_height=hb, max_height=hb)
            self.card3 = v.Card(flag=True, tile=True,
                                width=wr, min_width=wr, max_width=wr,
                                height=self._height, min_height=self._height, max_height=self._height)
            self.card4 = None
            
            self.card.children = [widgets.HBox([widgets.VBox([self.card1, self.card2]), self.card3])]
            if self.toggle_configure.value > 2: self.toggle_configure.value = 0
            self.toggle_configure.buttons[1].disabled = False
            self.toggle_configure.buttons[2].disabled = False
            self.toggle_configure.buttons[3].disabled = True
            
        # 4 contents
        else:
            self.card1 = v.Card(flag=True, tile=True, outlined=True, style_='border: 0px solid red; border-right: %dpx solid %s; border-bottom: %dpx solid %s; overflow: hidden;'%(self._borderwidth, self._color_first,
                                                                                                                                                                                   self._borderwidth, self._color_first),
                                width=wl, min_width=wl, max_width=wl,
                                height=ht, min_height=ht, max_height=ht)
            self.card2 = v.Card(flag=True, tile=True, outlined=True, style_='border: 0px solid red; border-right: %dpx solid %s; overflow: hidden;'%(self._borderwidth,self._color_first),
                                width=wl, min_width=wl, max_width=wl,
                                height=hb, min_height=hb, max_height=hb)
            self.card3 = v.Card(flag=True, tile=True, style_='border: 0px solid red; border-bottom: %dpx solid %s; overflow: hidden;'%(self._borderwidth,self._color_first),
                                width=wr, min_width=wr, max_width=wr,
                                height=ht, min_height=ht, max_height=ht)
            self.card4 = v.Card(flag=True, tile=True,
                                width=wr, min_width=wr, max_width=wr,
                                height=hb, min_height=hb, max_height=hb)
            self.card.children = [widgets.HBox([widgets.VBox([self.card1, self.card2]), widgets.VBox([self.card3, self.card4])])]
            self.toggle_configure.buttons[1].disabled = False
            self.toggle_configure.buttons[2].disabled = False
            self.toggle_configure.buttons[3].disabled = False
    
        
        self.updateColors()
        
        self.set1(self.card1children)
        self.set2(self.card2children)
        self.set3(self.card3children)
        self.set4(self.card4children)
    
    
    # Set the content of cards 1,2,3,4 (can pass None or a widget that has width and height properties
    def set1(self, children=None):
        self.card1children = children
        if self.card1 is not None:
            if self.card1children is None:
                self.card1.children = []
            else:
                self.card1children.width  = self.card1.width
                self.card1children.height = self.card1.height
                self.card1.children = [self.card1children.draw()]
                self.updateColors()
    
    def set2(self, children=None):
        self.card2children = children
        if self.card2 is not None:
            if self.card2children is None:
                self.card2.children = []
            else:
                self.card2children.width  = self.card2.width
                self.card2children.height = self.card2.height
                self.card2.children = [self.card2children.draw()]
                self.updateColors()
    
    def set3(self, children=None):
        self.card3children = children
        if self.card3 is not None:
            if self.card3children is None:
                self.card3.children = []
            else:
                self.card3children.width  = self.card3.width
                self.card3children.height = self.card3.height
                self.card3.children = [self.card3children.draw()]
                self.updateColors()
    
    def set4(self, children=None):
        self.card4children = children
        if self.card4 is not None:
            if self.card4children is None:
                self.card4.children = []
            else:
                self.card4children.width  = self.card4.width
                self.card4children.height = self.card4.height
                self.card4.children = [self.card4children.draw()]
                self.updateColors()
                
                
    #####################################################################################################################################################
    # Properties
    #####################################################################################################################################################
    
    @property
    def width(self):
        return self._width
        
    @width.setter
    def width(self, w):
        self._width = w
        self.card.width = self._width
        self.card.min_width = self._width
        self.card.max_width = self._width
        self.update()

        
    @property
    def height(self):
        return self._height
        
    @height.setter
    def height(self, h):
        self._height = h
        self.card.height = self._height
        self.card.min_height = self._height
        self.card.max_height = self._height
        self.update()

        
    @property
    def splitmode(self):
        return self._splitmode
        
    @splitmode.setter
    def splitmode(self, sm):
        self._splitmode = int(sm)
        
        self.select2.disabled = self._splitmode == 0
        self.select3.disabled = self._splitmode < 2
        self.select3.disabled = self._splitmode < 3
        self.select4.disabled = self._splitmode < 4
        
        self.sliderleftwidth.slider.disabled = self._splitmode == 0 or self._splitmode == 2
        self.slidertopheight.slider.disabled = self._splitmode < 2

        if self.updateWidgetsOnLoadState:
            old = self.toggle_splitmode.onchange
            self.toggle_splitmode.onchange = None
            self.toggle_splitmode.value = self._splitmode
            self.toggle_splitmode.onchange = old

        self.update()

        
    @property
    def leftwidthperc(self):
        return self._leftwidthperc
        
    @leftwidthperc.setter
    def leftwidthperc(self, w):
        self._leftwidthperc = min(100, max(w, 0))
        
        if self.updateWidgetsOnLoadState:
            old = self.sliderleftwidth.onchange
            self.sliderleftwidth.onchange = None
            self.sliderleftwidth.value = self._leftwidthperc
            self.sliderleftwidth.onchange = old
        
        self.update()

        
    @property
    def topheightperc(self):
        return self._topheightperc
        
    @topheightperc.setter
    def topheightperc(self, h):
        self._topheightperc = min(100, max(h, 0))
        
        if self.updateWidgetsOnLoadState:
            old = self.slidertopheight.onchange
            self.slidertopheight.onchange = None
            self.slidertopheight.value = self._topheightperc
            self.slidertopheight.onchange = old
        
        self.update()
        
        
    @property
    def borderwidth(self):
        return self._borderwidth
        
    @borderwidth.setter
    def borderwidth(self, w):
        self._borderwidth = min(3, max(int(w), 0))
        if self.propagateBounds:
            self.selectborder.value = str(self._borderwidth)
            
        if self.updateWidgetsOnLoadState:
            old = self.selectborder.onchange
            self.selectborder.onchange = None
            self.selectborder.value = str(self._borderwidth)
            self.selectborder.onchange = old
            
        self.update()

        
    @property
    def linkedmaps(self):
        return self._linkedmaps
        
    @linkedmaps.setter
    def linkedmaps(self, flag):
        self._linkedmaps = flag
        self.mapslinked.value = self._linkedmaps

        
    @property
    def color_first(self):
        return self._color_first
        
    @color_first.setter
    def color_first(self, color):
        self._color_first = color

        self.toggle_splitmode.colorselected = self._color_first
        self.select1.color = self._color_first
        self.select2.color = self._color_first
        self.select3.color = self._color_first
        self.select4.color = self._color_first
        self.sliderleftwidth.color = self._color_first
        self.slidertopheight.color = self._color_first
        self.mapslinked.color = self._color_first
        self.toggle_configure.colorselected = self._color_first
        self.selectborder.color = self._color_first
        self.update()


    @property
    def color_second(self):
        return self._color_second
        
    @color_second.setter
    def color_second(self, color):
        self._color_second = color

        self.toggle_splitmode.colorunselected = self._color_second
        self.toggle_configure.colorunselected = self._color_second
        self.updateColors()

        
    @property
    def dark(self):
        return self._dark
        
    @dark.setter
    def dark(self, flag):
        self._dark = flag
        
        self.toggle_splitmode.dark = self._dark
        self.toggle_configure.dark = self._dark
        self.updateColors()

        
    @property
    def state(self):
        s = {x: getattr(self, x) for x in ['width',
                                           'height',
                                           'splitmode',
                                           'leftwidthperc',
                                           'topheightperc',
                                           'borderwidth',
                                           'linkedmaps',
                                           'color_first',
                                           'color_second',
                                           'dark'
                                          ]}
        
        if self.card1children is not None: s['content1'] = self.card1children.state
        if self.card2children is not None: s['content2'] = self.card2children.state
        if self.card3children is not None: s['content3'] = self.card3children.state
        if self.card4children is not None: s['content4'] = self.card4children.state
        return s
        
    @state.setter
    def state(self, statusdict):
        self.showWaitDialogOnUpdate = False
        dlg = dialogWait.dialogWait(text='Updating content...', output=self.output, color=statusdict['color_first'], dark=statusdict['dark'])
        
        self.updateWidgetsOnLoadState = True
        
        self.card1children = self.card2children = self.card3children = self.card4children = None
        
        for key, value in statusdict.items():
            if key == 'content1':
                self.select1.value = value['content']
                self.card1children.state = value
            elif key == 'content2':
                self.select2.value = value['content']
                self.card2children.state = value
            elif key == 'content3':
                self.select3.value = value['content']
                self.card3children.state = value
            elif key == 'content4':
                self.select4.value = value['content']
                self.card4children.state = value
            else:
                setattr(self, key, value)
                
        self.update()
        
        self.updateWidgetsOnLoadState = False
        
        self.createAccessStrings()
        
        # Call the callback function
        if self.onStateChanged is not None:
            self.onStateChanged()
        
        dlg.close()
        self.showWaitDialogOnUpdate = True
        